package es.imib.cccf.beans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.primefaces.model.LazyDataModel;
import org.primefaces.model.SortOrder;

public abstract class PrimefacesList<T, P, D>
        extends LazyDataModel<T> implements Serializable {
    protected String sortField;
    protected boolean sortOrder;    
    protected Integer startRow;
    protected Integer maxResults;
    protected P persistenceBean;
    protected D dao;        
    protected Object[] parameters;        

    public PrimefacesList(D dao) {
        this.dao = dao;
    }

    public PrimefacesList(D dao, String sortField, boolean sortOrder) {
        this.sortField = sortField;
        this.sortOrder = sortOrder;
        this.dao = dao;
    }

    public PrimefacesList(D dao, Integer startRow, Integer maxResults) {
        this.startRow = startRow;
        this.maxResults = maxResults;        
        this.dao = dao;
    }

    public PrimefacesList(D dao, String sortField, boolean sortOrder, Integer startRow, Integer maxResults) {
        this.sortField = sortField;        
        this.sortOrder = sortOrder;
        this.startRow = startRow;
        this.maxResults = maxResults;
        this.dao = dao;        
    }

    protected List<T> transformarPersistenceCollectionToBeanCollection(List<P> items) {
        List<T> result = new ArrayList<>();
        for (int i = 0; i < items.size(); i++) {
            P p = items.get(i);
            T t = createBean(p);
            result.add(t);
        }
        return result;
    }

    public abstract void buscar();

    public abstract void listarTodos();

    protected abstract T createBean(P persistence);

    protected abstract int findTotalResults();

    protected abstract List<T> findBeans(String sortName, boolean ascending, Integer startRow, Integer maxResults);

    @Override
    public List<T> load(int first, int pageSize, String sortName, SortOrder sortOrder, Map map) {
        if ((startRow == null) || (maxResults == null)) {
            if (sortName != null) {
                if ((sortOrder != null) && (sortOrder.equals(SortOrder.DESCENDING))) {
                    return findBeans(sortName, false, first, pageSize);
                } else {
                    return findBeans(sortName, true, first, pageSize);
                }
            } else {
                if ((sortOrder != null) && (sortOrder.equals(SortOrder.DESCENDING))) {
                    return findBeans(sortField, false, first, pageSize);
                } else {
                    return findBeans(sortField, true, first, pageSize);
                }
            }
        } else {
            if (sortName != null) {
                if ((sortOrder != null) && (sortOrder.equals(SortOrder.DESCENDING))) {
                    return findBeans(sortName, false, startRow, maxResults);
                } else {
                    return findBeans(sortName, true, startRow, maxResults);
                }
            } else {
                if ((sortOrder != null) && (sortOrder.equals(SortOrder.DESCENDING))) {
                    return findBeans(sortField, false, startRow, maxResults);
                } else {
                    return findBeans(sortField, true, startRow, maxResults);
                }
            }
        }
    }

    @Override
    public void setRowIndex(int rowIndex) {
        if (rowIndex == -1 || getPageSize() == 0) {
            super.setRowIndex(-1);
        } else {
            super.setRowIndex(rowIndex % getPageSize());
        }
    }

    @Override
    public int getRowCount() {
        return findTotalResults();
    }

    @Override
    public abstract Object getRowKey(T object);

    public Integer getMaxResults() {
        return maxResults;
    }

    public void setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
    }
}
