package es.imib.cccf.beans;

import es.imib.cccf.semantic.collections.beans.SemanticBeanCollection;
import es.imib.cccf.util.Util;
import es.imib.cccf.web.application.OGIFilter;
import es.imib.export.ClassToOWL;
import es.imib.semantic.RDFQuery;
import es.imib.semantic.Tuple;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.openrdf.model.URI;
import org.primefaces.model.LazyDataModel;
import org.primefaces.model.SortOrder;
import org.semanticweb.owlapi.model.OWLClass;

public abstract class PrimefacesSemanticList<T extends SemanticBeanCollection>
        extends LazyDataModel<T> implements Serializable {

    protected RDFQuery rdfQuery;
    protected ClassToOWL classToOWL;
    protected ClassToOWL icd10ToOWL;
    protected String classFather;
    protected List<OWLClass> owlcs;
    protected List<OGIFilter> filters;
    protected Integer numResults;

    public PrimefacesSemanticList(String classFather) {
        rdfQuery = new RDFQuery(Util.getRepositoryRDF().getRepository());
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        icd10ToOWL = Util.getOntologyOWL().getIcd10ToOWL();
        this.classFather = classFather;
        OWLClass oc = classToOWL.getClass(classFather);
        owlcs = classToOWL.getAllSubclasses(oc);
        owlcs.add(oc);
        numResults = findTotalResults();
    }

    public Integer getNumResults() {
        return numResults;
    }

    protected List<T> transformarPersistenceCollectionToBeanCollection(List<URI> items,
            List<Tuple> tuples) {
        List<T> result = new ArrayList<>();
        for (URI u : items) {
            T t = createBean(u, tuples);
            result.add(t);
        }
        return result;
    }

    public abstract void buscar();

    public abstract void listarTodos();

    protected abstract T createBean(URI uri, List<Tuple> tuples);

    protected int findTotalResults() {
        return rdfQuery.countURIByClass(owlcs, filters);
    }

    protected List<T> findBeans(Integer offset, Integer size) {
        List<URI> uris = rdfQuery.findURIByClass(owlcs, filters, offset, size);
        List<Tuple> tuples = rdfQuery.runQueryTuplesByURI(uris);
        return transformarPersistenceCollectionToBeanCollection(uris, tuples);
    }

    @Override
    public List<T> load(int first, int pageSize, String sortName, SortOrder sortOrder, Map map) {
        return findBeans(first, pageSize);
    }

    @Override
    public void setRowIndex(int rowIndex) {
        if (rowIndex == -1 || getPageSize() == 0) {
            super.setRowIndex(-1);
        } else {
            super.setRowIndex(rowIndex % getPageSize());
        }
    }

    @Override
    public int getRowCount() {
        //numResults =  findTotalResults();
        return findTotalResults();
    }

    @Override
    public Object getRowKey(T object) {
        return object.getUri();
    }

    public void addFilter(String filter) {
        if (filters == null) {
            filters = new ArrayList<>();
        }
        if (filter != null && !filter.trim().equals("")) {
            OGIFilter of = new OGIFilter(filter, false);
            filters.add(of);
        }
    }
    
    public void addFilter(String filter, boolean condition) {
        if (filters == null) {
            filters = new ArrayList<>();
        }
        if (filter != null && !filter.trim().equals("")) {
            OGIFilter of = new OGIFilter(filter, condition);
            filters.add(of);
        }
    }
}
