package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.semantic.Tuple;
import java.util.Date;
import java.util.List;
import org.openrdf.model.URI;
import org.primefaces.model.chart.MeterGaugeChartModel;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class ConditionBean extends SemanticBean implements Comparable<ConditionBean> {

    //Data properties

    public static String REFERENCEDATE = "referenceDate";
    //Object properties
    public static String ASACLASSIFICACION = "hasASA_Classification";
    public static String MENOPAUSESTATUS = "hasMenopauseStatus";
    public static String KARNOFSKYSCORING = "hasKarnofskyScoring";
    //Properties
    protected Integer age;
    protected Date referenceDate;
    protected String asaClassification;
    protected String menopauseStatus;
    protected String karnofskyScoring;
    protected String designation;
    protected MeterGaugeChartModel chart;

    public ConditionBean(URI uri, String type) {
        super(uri, type);
        OWLDataProperty referenceDateProperty = classToOWL.getDataProperty(REFERENCEDATE);
        OWLObjectProperty hasASA_Classificacion = classToOWL.getObjetProperty(ASACLASSIFICACION);
        OWLObjectProperty hasMenopauseStatus = classToOWL.getObjetProperty(MENOPAUSESTATUS);
        OWLObjectProperty hasKarnofskyScoring = classToOWL.getObjetProperty(KARNOFSKYSCORING);
        List<Tuple> tuples = rdfQuery.runQueryTuplesByURI(uri);
        for (Tuple t : tuples) {
            if (t.getPredicate().stringValue().equals(referenceDateProperty.toStringID())) {
                String s = t.getObject().stringValue();
                referenceDate = Util.convertXMLDate(s);
            } else if (t.getPredicate().stringValue().equals(hasASA_Classificacion.toStringID())) {
                URI u = (URI) t.getObject();
                asaClassification = u.getLocalName();
            } else if (t.getPredicate().stringValue().equals(hasMenopauseStatus.toStringID())) {
                URI u = (URI) t.getObject();
                menopauseStatus = u.getLocalName();
            } else if (t.getPredicate().stringValue().equals(hasKarnofskyScoring.toStringID())) {
                URI u = (URI) t.getObject();
                karnofskyScoring = u.getLocalName().replaceAll("IND_", "");
            } else if (t.getPredicate().stringValue().equals(classToOWL.getDataProperty("designation").toStringID())) {
                String s = t.getObject().stringValue();
                designation = s;
            }
        }
        chart = new MeterGaugeChartModel();
        if (karnofskyScoring != null) {
            try {
                Integer ks = Integer.parseInt(karnofskyScoring.replaceAll("KS", ""));
                chart.addInterval(25);
                chart.addInterval(50);
                chart.addInterval(75);
                chart.addInterval(100);
                chart.setSeriesColors("cc6666,E7E658,93b75f,66cc66");
                chart.setValue(ks);
                chart.setGaugeLabel("Karnofsky Index");
                chart.setGaugeLabelPosition("bottom");
                chart.setShowTickLabels(false);                
            } catch (Exception ex) {

            }
        }
    }

    public MeterGaugeChartModel getChart() {
        return chart;
    }

    public String getDesignation() {
        return designation;
    }

    public String getKarnofskyScoring() {
        return karnofskyScoring;
    }

    public Integer getAge() {
        return age;
    }

    public Date getReferenceDate() {
        return referenceDate;
    }

    public String getAsaClassification() {
        return asaClassification;
    }

    public String getMenopauseStatus() {
        return menopauseStatus;
    }

    @Override
    public int compareTo(ConditionBean o) {
        return this.referenceDate.compareTo(o.getReferenceDate());
    }
}
