package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.semantic.Tuple;
import java.util.Date;
import java.util.List;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class CourseBean extends SemanticDateBean implements Comparable<CourseBean> {
    //Data properties
    public static String EXAMINATIONDATE = "examinationDate";
    //Object properties
    public static String CONDITION = "hasCondition";
    //Propeties
    protected ConditionBean condition;
    protected Date examinationDate;
    protected DiagnosisBean diagnosis;
    protected OWLDataProperty examinationDateProperty;
    protected OWLObjectProperty hasConditionProperty;
    protected Integer chartValue;

    public CourseBean() {
    }

    public CourseBean(PatientBean patient, URI uri, String type, DiagnosisBean diagnosis) {
        super(patient, uri, type);        
        this.diagnosis = diagnosis;
        examinationDateProperty = classToOWL.getDataProperty(EXAMINATIONDATE);
        hasConditionProperty = classToOWL.getObjetProperty(CONDITION);
        List<Tuple> tuples = rdfQuery.runQueryTuplesByURI(uri);
        loadSemanticInformation(tuples);
    }

    public ConditionBean getCondition() {
        return condition;
    }

    public Date getExaminationDate() {
        return examinationDate;
    }
    
    public String getType() {
        if (types == null) {
            return "";
        }
        if (types.isEmpty()) {
            return "";
        }
        return types.get(0).getIRI().getFragment();
    }

    @Override
    protected void loadOtherSemanticInformation(Tuple t) {
        if (t.getPredicate().stringValue().equals(examinationDateProperty.toStringID())) {
            String s = t.getObject().stringValue();
            examinationDate = Util.convertXMLDate(s);
        } else if (t.getPredicate().stringValue().equals(hasConditionProperty.toStringID())) {
            URI u = (URI) t.getObject();
            for (ConditionBean cb : patient.getConditions()) {
                if (u.toString().equals(cb.getUri().toString())) {
                    condition = cb;
                    break;
                }
            }
        }
        if (!types.isEmpty()) {
            OWLClass oc = types.get(0);
            String label = classToOWL.getAnnotationPropertyOwlEntity(classToOWL.getAnnotationProperty("chartValue"), oc);
            try {
                chartValue = Integer.parseInt(label);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    public Integer getChartValue() {
        return chartValue;
    }

    @Override
    public int compareTo(CourseBean o) {
        return this.examinationDate.compareTo(o.getExaminationDate());
    }

    public DiagnosisBean getDiagnosis() {
        return diagnosis;
    }
}
