package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.semantic.Tuple;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class DiagnosisBean extends SemanticDateBean implements Comparable<DiagnosisBean> {    
    // Data propeties
    public static String REFERENCEDATE = "referenceDate";    
    // Object properties
    public static String THERAPY = "hasTherapy";
    public static String COURSE = "hasDiseaseCourse";
    public static String ICD10 = "hasICD10_Classification";
    public static String GRADING = "hasGrading";
    public static String ANATOMICALSTRUCTURE = "hasAnatomicalStructure";
    public static String PATHOLOGICALSTRUCTURE = "hasPathologicalStructure";
    public static String TUMORTYPE = "hasTumorType";
    public static String PATHOLOGICAL_T = "hasPathologicalPrimaryTumor";
    public static String PATHOLOGICAL_N = "hasPathologicalRegionalLymphNodes";
    public static String PATHOLOGICAL_M = "hasPathologicalDistantMetastasis";
    public static String PATHOLOGICAL_STAGE = "hasPathologicalStage";
    public static String CLINICAL_T = "hasClinicalPrimaryTumor";
    public static String CLINICAL_N = "hasClinicalRegionalLymphNodes";
    public static String CLINICAL_M = "hasClinicalDistantMetastasis";
    public static String CLINICAL_STAGE = "hasClinicalStage";
            
    // Properties
    protected List<TherapyBean> therapies;
    protected List<CourseBean> courses;    
    protected Date referenceDate;
    protected String icd10Code;
    protected List<String> icd10Description;
    protected OWLDataProperty referenceDateProperty;
    protected OWLObjectProperty hasTherapyProperty;
    protected OWLObjectProperty hasDiseaseCourseProperty;
    protected OWLObjectProperty hasICD10_ClassificationProperty;
    protected OWLObjectProperty hasGrading;
    protected OWLObjectProperty hasAnatomicalStructure;
    protected OWLObjectProperty hasPathologicalStructure;
    protected OWLObjectProperty hasTumorType;
    protected OWLObjectProperty hasPathologicalPrimaryTumor;
    protected OWLObjectProperty hasPathologicalRegionalLymphNodes;
    protected OWLObjectProperty hasPathologicalDistantMetastasis;
    protected OWLObjectProperty hasPathologicalStage;
    protected OWLObjectProperty hasClinicalPrimaryTumor;
    protected OWLObjectProperty hasClinicalRegionalLymphNodes;
    protected OWLObjectProperty hasClinicalDistantMetastasis;
    protected OWLObjectProperty hasClinicalStage;
    protected String organ;
    protected String tumorType;
    protected String cancerType;
    protected String grading;
    protected String pathologicalT;
    protected String pathologicalN;
    protected String pathologicalM;
    protected String pathologicalStage;
    protected String clinicalT;
    protected String clinicalN;
    protected String clinicalM;
    protected String clinicalStage;

    public DiagnosisBean() {
    }

    public DiagnosisBean(PatientBean patient, URI uri, String type) {
        super(patient, uri, type);        
        therapies = new ArrayList<>();
        courses = new ArrayList<>();
        referenceDateProperty = classToOWL.getDataProperty(REFERENCEDATE);
        hasTherapyProperty = classToOWL.getObjetProperty(THERAPY);
        hasDiseaseCourseProperty = classToOWL.getObjetProperty(COURSE);
        hasICD10_ClassificationProperty = classToOWL.getObjetProperty(ICD10);
        hasAnatomicalStructure = classToOWL.getObjetProperty(ANATOMICALSTRUCTURE);
        hasGrading = classToOWL.getObjetProperty(GRADING);
        hasPathologicalStructure = classToOWL.getObjetProperty(PATHOLOGICALSTRUCTURE);
        hasTumorType = classToOWL.getObjetProperty(TUMORTYPE);
        hasPathologicalDistantMetastasis = classToOWL.getObjetProperty(PATHOLOGICAL_M);
        hasPathologicalPrimaryTumor = classToOWL.getObjetProperty(PATHOLOGICAL_T);
        hasPathologicalStage = classToOWL.getObjetProperty(PATHOLOGICAL_STAGE);
        hasPathologicalRegionalLymphNodes = classToOWL.getObjetProperty(PATHOLOGICAL_N);
        hasClinicalDistantMetastasis = classToOWL.getObjetProperty(CLINICAL_M);
        hasClinicalPrimaryTumor = classToOWL.getObjetProperty(CLINICAL_T);
        hasClinicalStage = classToOWL.getObjetProperty(CLINICAL_STAGE);
        hasClinicalRegionalLymphNodes = classToOWL.getObjetProperty(CLINICAL_N);
        List<Tuple> tuples = rdfQuery.runQueryTuplesByURI(uri);        
        loadSemanticInformation(tuples);
    }

    public String getPathologicalT() {
        return pathologicalT;
    }

    public String getPathologicalN() {
        return pathologicalN;
    }

    public String getPathologicalM() {
        return pathologicalM;
    }

    public String getPathologicalStage() {
        return pathologicalStage;
    }

    public String getClinicalT() {
        return clinicalT;
    }

    public String getClinicalN() {
        return clinicalN;
    }

    public String getClinicalM() {
        return clinicalM;
    }

    public String getClinicalStage() {
        return clinicalStage;
    }

    public List<TherapyBean> getTherapies() {
        return therapies;
    }

    public List<CourseBean> getCourses() {
        return courses;
    }

    public Date getReferenceDate() {
        return referenceDate;
    }

    public String getIcd10Code() {
        return icd10Code;
    }

    public List<String> getIcd10Description() {
        return icd10Description;
    }

    @Override
    protected void loadOtherSemanticInformation(Tuple t) {
        if (t.getPredicate().stringValue().equals(referenceDateProperty.toStringID())) {
            String s = t.getObject().stringValue();
            referenceDate = Util.convertXMLDate(s);
        } else if (t.getPredicate().stringValue().equals(hasTherapyProperty.toStringID())) {
            URI u = (URI) t.getObject();
            TherapyBean tb = new TherapyBean(patient, u, "Therapy", this);
            therapies.add(tb);
        } else if (t.getPredicate().stringValue().equals(hasDiseaseCourseProperty.toStringID())) {
            URI u = (URI) t.getObject();
            CourseBean cb = new CourseBean(patient, u, "DiseaseCourse", this);
            courses.add(cb);
        } else if (t.getPredicate().stringValue().equals(hasICD10_ClassificationProperty.toStringID())) {
            URI u = (URI) t.getObject();
            OWLClass oc = icd10ToOWL.getClass(u.getLocalName().replaceAll("IND_", ""));
            OWLAnnotationProperty hasDescription = icd10ToOWL.getAnnotationProperty("has_Description");
            icd10Description = icd10ToOWL.getAnnotationPropertyValue(hasDescription, oc);            
            icd10Code = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasAnatomicalStructure.toStringID())) {
            URI u = (URI) t.getObject();
            organ = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasGrading.toStringID())) {
            URI u = (URI) t.getObject();
            grading = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasPathologicalStructure.toStringID())) {
            URI u = (URI) t.getObject();
            cancerType = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasTumorType.toStringID())) {
            URI u = (URI) t.getObject();
            tumorType = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasPathologicalDistantMetastasis.toStringID())) {
            URI u = (URI) t.getObject();
            pathologicalM = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasPathologicalRegionalLymphNodes.toStringID())) {
            URI u = (URI) t.getObject();
            pathologicalN = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasPathologicalPrimaryTumor.toStringID())) {
            URI u = (URI) t.getObject();
            pathologicalT = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasPathologicalStage.toStringID())) {
            URI u = (URI) t.getObject();
            pathologicalStage = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasClinicalDistantMetastasis.toStringID())) {
            URI u = (URI) t.getObject();
            clinicalM = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasClinicalRegionalLymphNodes.toStringID())) {
            URI u = (URI) t.getObject();
            clinicalN = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasClinicalPrimaryTumor.toStringID())) {
            URI u = (URI) t.getObject();
            clinicalT = u.getLocalName().replaceAll("IND_", "");
        } else if (t.getPredicate().stringValue().equals(hasClinicalStage.toStringID())) {
            URI u = (URI) t.getObject();
            clinicalStage = u.getLocalName().replaceAll("IND_", "");
        }
    }

    public String getGrading() {
        return grading;
    }

    @Override
    public int compareTo(DiagnosisBean o) {
        return this.referenceDate.compareTo(o.getReferenceDate());
    }

    public String getOrgan() {
        return organ;
    }

    public String getTumorType() {
        return tumorType;
    }

    public String getCancerType() {
        return cancerType;
    }
}

