package es.imib.cccf.semantic.beans;

import es.imib.cccf.beans.PrimefacesSemanticList;
import es.imib.cccf.semantic.collections.GenericList;
import es.imib.cccf.util.Util;
import es.imib.cccf.web.application.OGIClass;
import es.imib.cccf.web.application.OGIFilter;
import es.imib.cccf.web.application.OGIStatement;
import es.imib.cccf.web.application.OntologyGuidedInputCache;
import es.imib.cccf.web.application.OntologyOWL;
import es.imib.jsf.JSFUtil;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import org.semanticweb.owlapi.model.OWLClass;

@ManagedBean
@ViewScoped
public class OntologyGuidedInputWeb implements Serializable {

    protected OntologyGuidedInputCache ogiCache;
    protected OGIClass firstClass;
    protected OntologyOWL ontologyOWL;
    protected List<OGIStatement> statements;
    protected List<OGIStatement> allStatements;
    protected String mainVariable = "?s";
    protected String prefixVariable = "?a";
    protected Integer variableNum = 0;
    protected String classSelected;
    protected String fragment;
    protected OGIClass parameterClass;
    protected List<OGIClass> ogics;
    protected boolean error;
    protected PrimefacesSemanticList list;

    public OntologyGuidedInputWeb() {
        ogiCache = Util.getOntologyGuidedInputCache();
        ontologyOWL = Util.getOntologyOWL();
        statements = new ArrayList<>();
        allStatements = new ArrayList<>();
        ogics = new ArrayList<>();
        fragment = JSFUtil.getStringHttpRequestParameter("f");
        if (fragment == null) {
            error = true;
            return;
        }
        OWLClass oc = ontologyOWL.getClassToOWL().getClass(fragment);
        for (OGIClass ogic : ogiCache.getClasses()) {
            if (ogic.getUri().equals(oc.toStringID())) {
                parameterClass = ogic;
                break;
            }
        }
        if (parameterClass == null) {
            error = true;
            return;
        }
        ogics.add(parameterClass);
        List<OWLClass> ocs = ontologyOWL.getClassToOWL().getAllSubclasses(oc);
        for (OWLClass o : ocs) {
            for (OGIClass ogic : ogiCache.getClasses()) {
                if (o.toStringID().equals(ogic.getUri())) {
                    ogics.add(ogic);
                }
            }
        }
        classSelected = parameterClass.getUri();
        list = new GenericList(fragment);
    }
    
    public OntologyGuidedInputWeb(String fragment) {
        ogiCache = Util.getOntologyGuidedInputCache();
        ontologyOWL = Util.getOntologyOWL();
        statements = new ArrayList<>();
        allStatements = new ArrayList<>();
        ogics = new ArrayList<>();
        this.fragment = fragment;
        OWLClass oc = ontologyOWL.getClassToOWL().getClass(fragment);
        for (OGIClass ogic : ogiCache.getClasses()) {
            if (ogic.getUri().equals(oc.toStringID())) {
                parameterClass = ogic;
                break;
            }
        }
        if (parameterClass == null) {
            error = true;
            return;
        }
        ogics.add(parameterClass);
        List<OWLClass> ocs = ontologyOWL.getClassToOWL().getAllSubclasses(oc);
        for (OWLClass o : ocs) {
            for (OGIClass ogic : ogiCache.getClasses()) {
                if (o.toStringID().equals(ogic.getUri())) {
                    ogics.add(ogic);
                }
            }
        }
        classSelected = parameterClass.getUri();        
        generateList();
    }
    
    protected void generateList() {
        list = new GenericList(fragment);
    }

    public PrimefacesSemanticList getList() {
        return list;
    }

    public String getFragment() {
        return fragment;
    }

    public OGIClass getParameterClass() {
        return parameterClass;
    }

    public List<OGIClass> getOgics() {
        return ogics;
    }

    public boolean isError() {
        return error;
    }

    public String getClassSelected() {
        return classSelected;
    }

    public void setClassSelected(String classSelected) {
        this.classSelected = classSelected;
    }

    public List<OGIStatement> getStatements() {
        return statements;
    }

    public OGIClass getFirstClass() {
        return firstClass;
    }

    public void addFirstStatement() {
        if (classSelected == null) {
            return;
        }
        for (OGIClass oc : ogiCache.getClasses()) {
            if (classSelected.equals(oc.getUri())) {
                firstClass = oc;
                break;
            }
        }
        if (firstClass == null) {
            return;
        }
        statements = new ArrayList<>();
        allStatements = new ArrayList<>();
        OGIStatement statement = new OGIStatement(null, statements, "a", mainVariable, firstClass, allStatements, variableNum, 0, true);
        variableNum++;
        statements.add(statement);
        allStatements.add(statement);
    }

    public void addStatement() {
        if (classSelected == null) {
            return;
        }
        if (firstClass == null) {
            return;
        }
        OGIStatement statement = new OGIStatement(null, statements, "a", mainVariable, firstClass, allStatements, variableNum, 0, false);
        variableNum++;
        statements.add(statement);
        allStatements.add(statement);
    }

    public List<OGIStatement> getAllStatements() {
        return allStatements;
    }

    public String generateSPARQL() {
        String sparql = "SELECT " + mainVariable + " WHERE { ";
        sparql += mainVariable + " rdf:type " + Util.generateFilterTypeOWL(mainVariable, ontologyOWL.getClassToOWL(), classSelected);
        if (!statements.isEmpty()) {
            sparql += " . { ";
            for (OGIStatement st : statements) {
                sparql += st.getSPARQL();
            }
            sparql += " } ";
        }

        sparql += " } ";
        return sparql;
    }

    public void search() {
        if (classSelected == null) {
            return;
        }
        for (OGIClass oc : ogiCache.getClasses()) {
            if (classSelected.equals(oc.getUri())) {
                firstClass = oc;
                break;
            }
        }
        if (firstClass == null) {
            return;
        }
        generateSearchList(firstClass.getFragment());
        if (!statements.isEmpty()) {
            for (OGIStatement st : allStatements) {
                list.addFilter(st.getSPARQL(), st.isCondition());
            }
        }
        list.buscar();
    }
    
    protected List<OGIFilter> generateFilters() {
        List<OGIFilter> result = new ArrayList<>();
        if (allStatements == null) {
            return result;
        }
        for (OGIStatement st : allStatements) {
            OGIFilter of = new OGIFilter(st.getSPARQL(), st.isCondition());
            result.add(of);
        }
        return result;
    }
    
    protected void generateSearchList(String fragment) {
        list = new GenericList(fragment);
    }
    
    public void deleteStatement(int variableNum) {
        for (OGIStatement os : statements) {
            if (os.getVariableNum() == variableNum) {
                statements.remove(os);
                break;
            }
        }
        for (OGIStatement os : allStatements) {
            if (os.getVariableNum() == variableNum) {
                allStatements.remove(os);
                break;
            }
        }
    }
    
    public boolean isFirstStatement() {
        if (statements == null) {
            return true;
        }
        if (statements.isEmpty()) {
            return true;
        }
        return false;
    }
}
