package es.imib.cccf.semantic.beans;

import es.imib.semantic.Tuple;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import org.openrdf.model.URI;
import org.primefaces.model.chart.LineChartModel;
import org.primefaces.model.chart.LineChartSeries;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class PatientBean extends SemanticBean {

    // Data properties
    public static String AGE_PROPERTY = "age";
    public static String GENDER_PROPERTY = "gender";
    // Object properties
    public static String HAS_CONDITION = "hasCondition";
    public static String HAS_DIAGNOSIS = "hasDiagnosis";
    // Properties
    protected List<ConditionBean> conditions;
    protected List<DiagnosisBean> diagnosis;
    protected List<TherapyBean> therapies;
    protected List<CourseBean> courses;
    protected String gender;
    protected Integer age;
    protected int months;
    protected Date startDate;
    protected Date endDate;
    protected List<Integer> array;
    protected List<List<CourseBean>> coursesTable;
    protected TherapyBean therapySelected;
    protected DiagnosisBean diagnosisSelected;
    protected CourseBean courseSelected;
    protected LineChartModel lineChartModel;
    protected LineChartModel karnofskyChartModel;

    public PatientBean(URI uri, String type) {
        super(uri, type);
        therapySelected = new TherapyBean();
        diagnosisSelected = new DiagnosisBean();
        courseSelected = new CourseBean();
        diagnosis = new ArrayList<>();
        conditions = new ArrayList<>();
        courses = new ArrayList<>();
        therapies = new ArrayList<>();
        List<Tuple> tuples = rdfQuery.runQueryTuplesByURI(uri);
        if (tuples.isEmpty()) {
            return;
        }
        OWLDataProperty ageProperty = classToOWL.getDataProperty(AGE_PROPERTY);
        OWLDataProperty genderProperty = classToOWL.getDataProperty(GENDER_PROPERTY);
        OWLObjectProperty diagnosisProperty = classToOWL.getObjetProperty(HAS_DIAGNOSIS);
        OWLObjectProperty conditionProperty = classToOWL.getObjetProperty(HAS_CONDITION);
        for (Tuple tuple : tuples) {
            if (tuple.getPredicate().stringValue().equals(ageProperty.toStringID())) {
                try {
                    age = Integer.parseInt(tuple.getObject().stringValue());
                } catch (Exception ex) {

                }
            } else if (tuple.getPredicate().stringValue().equals(genderProperty.toStringID())) {
                gender = tuple.getObject().stringValue();
            } else if (tuple.getPredicate().stringValue().equals(diagnosisProperty.toStringID())) {
                URI u = (URI) tuple.getObject();
                DiagnosisBean db = new DiagnosisBean(this, u, "Diagnosis");
                diagnosis.add(db);
            } else if (tuple.getPredicate().stringValue().equals(conditionProperty.toStringID())) {
                URI u = (URI) tuple.getObject();
                ConditionBean cb = new ConditionBean(u, "PatientCondition");
                conditions.add(cb);
            }
        }
        loadTypes(tuples);
        for (DiagnosisBean d : diagnosis) {
            for (TherapyBean t : d.getTherapies()) {
                therapies.add(t);
            }
            for (CourseBean c : d.getCourses()) {
                courses.add(c);
            }
        }
        Collections.sort(diagnosis);
        Collections.sort(conditions);
        Collections.sort(therapies);
        Collections.sort(courses);
        if (therapies.isEmpty()) {
            months = 0;
        } else {
            startDate = therapies.get(0).getStartDate();
            TherapyBean last = therapies.get(therapies.size() - 1);
            if (last.getEndDate() != null) {
                endDate = last.getEndDate();
            } else {
                endDate = last.getStartDate();
            }
            for (TherapyBean tb : therapies) {
                if (tb.getEndDate() != null) {
                    if (endDate.compareTo(tb.getEndDate()) < 0) {
                        endDate = tb.getEndDate();
                    }
                } else {
                    if (endDate.compareTo(tb.getStartDate()) < 0) {
                        endDate = tb.getStartDate();
                    }
                }
            }
            if (!courses.isEmpty()) {
                CourseBean cb = courses.get(courses.size() - 1);
                if (cb.getExaminationDate().compareTo(endDate) > 0) {
                    endDate = cb.getExaminationDate();
                }
            }
            Calendar calendarStart = Calendar.getInstance();
            calendarStart.setTime(startDate);
            Calendar calendarEnd = Calendar.getInstance();
            calendarEnd.setTime(endDate);
            months = 0;
            while (calendarStart.get(Calendar.MONTH) != calendarEnd.get(Calendar.MONTH)
                    || calendarStart.get(Calendar.YEAR) != calendarEnd.get(Calendar.YEAR)) {
                months++;
                calendarStart.add(Calendar.MONTH, 1);
            }
            months++;
        }
        array = new ArrayList<>();
        for (int i = 0; i < months; i++) {
            array.add(i + 1);
        }
        for (TherapyBean tb : therapies) {
            tb.loadTimes();
        }
        if (!therapies.isEmpty()) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            coursesTable = new ArrayList<>();
            for (int i = 0; i < array.size(); i++) {
                List<CourseBean> cbs = new ArrayList<>();
                for (CourseBean cb : courses) {
                    Calendar referenceDate = Calendar.getInstance();
                    referenceDate.setTime(cb.getExaminationDate());
                    if (referenceDate.get(Calendar.YEAR) == calendar.get(Calendar.YEAR)) {
                        if (referenceDate.get(Calendar.MONTH) == calendar.get(Calendar.MONTH)) {
                            cbs.add(cb);
                        }
                    }
                }
                coursesTable.add(cbs);
                calendar.add(Calendar.MONTH, 1);
            }
        }

    }

    public PatientBean(String id, String type) {
        super(id, type);
        therapySelected = new TherapyBean();
        diagnosisSelected = new DiagnosisBean();
        courseSelected = new CourseBean();
        diagnosis = new ArrayList<>();
        conditions = new ArrayList<>();
        courses = new ArrayList<>();
        therapies = new ArrayList<>();
        OWLClass oc = classToOWL.getClass("Patient");
        List<Tuple> tuples = rdfQuery.runQueryTuplesById(id, idProperty.toStringID(), oc.toStringID());
        if (tuples.isEmpty()) {
            return;
        }
        uri = (URI) tuples.get(0).getSubject();
        OWLDataProperty ageProperty = classToOWL.getDataProperty(AGE_PROPERTY);
        OWLDataProperty genderProperty = classToOWL.getDataProperty(GENDER_PROPERTY);
        OWLObjectProperty diagnosisProperty = classToOWL.getObjetProperty(HAS_DIAGNOSIS);
        OWLObjectProperty conditionProperty = classToOWL.getObjetProperty(HAS_CONDITION);
        for (Tuple tuple : tuples) {
            if (tuple.getPredicate().stringValue().equals(ageProperty.toStringID())) {
                try {
                    age = Integer.parseInt(tuple.getObject().stringValue());
                } catch (Exception ex) {

                }
            } else if (tuple.getPredicate().stringValue().equals(genderProperty.toStringID())) {
                gender = tuple.getObject().stringValue();
            } else if (tuple.getPredicate().stringValue().equals(diagnosisProperty.toStringID())) {
                URI uri = (URI) tuple.getObject();
                DiagnosisBean db = new DiagnosisBean(this, uri, "Diagnosis");
                diagnosis.add(db);
            } else if (tuple.getPredicate().stringValue().equals(conditionProperty.toStringID())) {
                URI uri = (URI) tuple.getObject();
                ConditionBean cb = new ConditionBean(uri, "PatientCondition");
                conditions.add(cb);
            }
        }
        loadTypes(tuples);
        for (DiagnosisBean d : diagnosis) {
            for (TherapyBean t : d.getTherapies()) {
                therapies.add(t);
            }
            for (CourseBean c : d.getCourses()) {
                courses.add(c);
            }
        }
        Collections.sort(diagnosis);
        Collections.sort(conditions);
        Collections.sort(therapies);
        Collections.sort(courses);
        if (therapies.isEmpty()) {
            months = 0;
        } else {
            startDate = therapies.get(0).getStartDate();
            TherapyBean last = therapies.get(therapies.size() - 1);
            if (last.getEndDate() != null) {
                endDate = last.getEndDate();
            } else {
                endDate = last.getStartDate();
            }
            for (TherapyBean tb : therapies) {
                if (tb.getEndDate() != null) {
                    if (endDate.compareTo(tb.getEndDate()) < 0) {
                        endDate = tb.getEndDate();
                    }
                } else {
                    if (endDate.compareTo(tb.getStartDate()) < 0) {
                        endDate = tb.getStartDate();
                    }
                }
            }
            if (!courses.isEmpty()) {
                CourseBean cb = courses.get(courses.size() - 1);
                if (cb.getExaminationDate().compareTo(endDate) > 0) {
                    endDate = cb.getExaminationDate();
                }
            }
            Calendar calendarStart = Calendar.getInstance();
            calendarStart.setTime(startDate);
            Calendar calendarEnd = Calendar.getInstance();
            calendarEnd.setTime(endDate);
            months = 0;
            while (calendarStart.get(Calendar.MONTH) != calendarEnd.get(Calendar.MONTH)
                    || calendarStart.get(Calendar.YEAR) != calendarEnd.get(Calendar.YEAR)) {
                months++;
                calendarStart.add(Calendar.MONTH, 1);
            }
            months++;
        }
        array = new ArrayList<>();
        for (int i = 0; i < months; i++) {
            array.add(i + 1);
        }
        for (TherapyBean tb : therapies) {
            tb.loadTimes();
        }
        if (!therapies.isEmpty()) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            coursesTable = new ArrayList<>();
            for (int i = 0; i < array.size(); i++) {
                List<CourseBean> cbs = new ArrayList<>();
                for (CourseBean cb : courses) {
                    Calendar referenceDate = Calendar.getInstance();
                    referenceDate.setTime(cb.getExaminationDate());
                    if (referenceDate.get(Calendar.YEAR) == calendar.get(Calendar.YEAR)) {
                        if (referenceDate.get(Calendar.MONTH) == calendar.get(Calendar.MONTH)) {
                            cbs.add(cb);
                        }
                    }
                }
                coursesTable.add(cbs);
                calendar.add(Calendar.MONTH, 1);
            }
        }
        lineChartModel = new LineChartModel();
        LineChartSeries series = new LineChartSeries();
        series.setLabel("Patient evolution");
        int i = 1;
        for (CourseBean cb : courses) {
            if (cb.getChartValue() != null) {
                series.set(i, cb.getChartValue());
            }
            i++;
        }
        lineChartModel.addSeries(series);
        lineChartModel.setAnimate(true);
        karnofskyChartModel = new LineChartModel();
        series = new LineChartSeries();
        series.setLabel("Karnofsky index evolution");
        i = 1;
        for (ConditionBean cb : conditions) {
            if (cb.getKarnofskyScoring() != null) {
                try {
                    series.set(i, Integer.parseInt(cb.getKarnofskyScoring().replace("KS", "")));
                } catch (Exception ex) {

                }
            }
            i++;
        }
        series.setFill(true);
        karnofskyChartModel.setAnimate(true);
        karnofskyChartModel.addSeries(series);
    }

    public LineChartModel getLineChartModel() {
        return lineChartModel;
    }

    public LineChartModel getKarnofskyChartModel() {
        return karnofskyChartModel;
    }
    
    

    public List<List<CourseBean>> getCoursesTable() {
        return coursesTable;
    }

    public TherapyBean getTherapySelected() {
        return therapySelected;
    }

    public DiagnosisBean getDiagnosisSelected() {
        return diagnosisSelected;
    }

    public CourseBean getCourseSelected() {
        return courseSelected;
    }

    public int getMonths() {
        return months;
    }

    public List<Integer> getArray() {
        return array;
    }

    public Date getStartDate() {
        return startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public List<ConditionBean> getConditions() {
        return conditions;
    }

    public List<DiagnosisBean> getDiagnosis() {
        return diagnosis;
    }

    public String getGender() {
        return gender;
    }

    public Integer getAge() {
        return age;
    }

    public List<TherapyBean> getTherapies() {
        return therapies;
    }

    public List<CourseBean> getCourses() {
        return courses;
    }

    public void selectTherapy(String id) {
        for (TherapyBean tb : therapies) {
            if (tb.getId().equals(id)) {
                therapySelected = tb;
                break;
            }
        }
    }

    public void selectDiagnosis(String id) {
        for (DiagnosisBean tb : diagnosis) {
            if (tb.getId().equals(id)) {
                diagnosisSelected = tb;
                break;
            }
        }
    }

    public void selectCourse(String id) {
        for (CourseBean tb : courses) {
            if (tb.getId().equals(id)) {
                courseSelected = tb;
                break;
            }
        }
    }
}
