package es.imib.cccf.semantic.beans;

import es.imib.cccf.semantic.collections.OWLClassDescription;
import es.imib.cccf.util.Util;
import es.imib.cccf.web.application.OGIFilter;
import es.imib.export.ClassToOWL;
import es.imib.jsf.JSFUtil;
import es.imib.semantic.RDFQuery;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

@ManagedBean
@ViewScoped
public class RecommendBean implements Serializable {

    protected ClassToOWL classToOWL;
    protected ClassToOWL icd10ToOWL;
    protected RDFQuery rdfQuery;
    protected String age;
    protected String gender;
    protected List<OWLClassDescription> tumorTypes;
    protected List<OWLClassDescription> cancerTypes;
    protected List<OWLClassDescription> organs;
    protected List<OWLClassDescription> icd10diagnosis;
    protected List<OWLClassDescription> therapies;
    protected List<OWLClassDescription> courses;
    protected String tumorType;
    protected String cancerType;
    protected String organ;
    protected String icd10;
    protected String therapy;
    protected OWLObjectProperty hasDiagnosis;
    protected OWLObjectProperty hasTherapy;
    protected OWLObjectProperty hasDiseaseCourse;
    protected OWLObjectProperty hasICD10_Classification;
    protected OWLObjectProperty hasTumorType;
    protected OWLObjectProperty hasCancerType;
    protected OWLObjectProperty hasOrgan;
    protected OWLDataProperty hasAge;
    protected OWLDataProperty hasGender;
    protected Integer resultPatientsRecommendation;
    protected List<TherapyBean> therapiesBean;
    protected List<CourseBean> coursesBean;
    protected int months;
    protected String recommendationMonths;
    protected List<PieChartModelBean> therapiesBeanChart;
    protected List<PieChartModelBean> coursesBeanChart;
    protected List<PatientBean> pbs;
    protected int actualMonth;
    protected List<String> therapyTypes;
    
    public RecommendBean() {        
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        icd10ToOWL = Util.getOntologyOWL().getIcd10ToOWL();
        OWLAnnotationProperty owlap = classToOWL.getAnnotationProperty("http://www.w3.org/2000/01/rdf-schema", "label");
        OWLAnnotationProperty owlde = icd10ToOWL.getAnnotationProperty("has_Description");
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        icd10ToOWL = Util.getOntologyOWL().getIcd10ToOWL();
        tumorTypes = Util.getListOWLClassDescription("TumorType", classToOWL, false, owlap);
        cancerTypes = Util.getListOWLClassDescription("CancerType", classToOWL, false, owlap);
        organs = Util.getListOWLClassDescription("Organ", classToOWL, false, owlap);
        icd10diagnosis = Util.getListOWLClassDescription("C00_D48", icd10ToOWL, true, owlde);
        therapies = Util.getListOWLClassDescription("Therapy", classToOWL, true, owlap);
        courses = Util.getListOWLClassDescription("DiseaseCourse", classToOWL, true, owlap);
        hasDiagnosis = classToOWL.getObjetProperty("hasDiagnosis");
        hasDiseaseCourse = classToOWL.getObjetProperty("hasDiseaseCourse");
        hasTherapy = classToOWL.getObjetProperty("hasTherapy");
        hasCancerType = classToOWL.getObjetProperty("hasPathologicalStructure");
        hasTumorType = classToOWL.getObjetProperty("hasTumorType");
        hasOrgan = classToOWL.getObjetProperty("hasAnatomicalStructure");
        hasICD10_Classification = classToOWL.getObjetProperty("hasICD10_Classification");
        hasAge = classToOWL.getDataProperty("age");
        hasGender = classToOWL.getDataProperty("gender");
        rdfQuery = new RDFQuery(Util.getRepositoryRDF().getRepository());
        therapiesBeanChart = new ArrayList<>();
        coursesBeanChart = new ArrayList<>();
        months = 0;
        actualMonth = 1;
    }

    public int getActualMonth() {
        return actualMonth;
    }

    public String getRecommendationMonths() {
        return recommendationMonths;
    }

    public void setRecommendationMonths(String recommendationMonths) {
        this.recommendationMonths = recommendationMonths;
    }

    public String getAge() {
        return age;
    }

    public void setAge(String age) {
        this.age = age;
    }

    public String getGender() {
        return gender;
    }

    public void setGender(String gender) {
        this.gender = gender;
    }

    public List<OWLClassDescription> getTumorTypes() {
        return tumorTypes;
    }

    public List<OWLClassDescription> getCancerTypes() {
        return cancerTypes;
    }

    public List<OWLClassDescription> getOrgans() {
        return organs;
    }

    public List<OWLClassDescription> getIcd10diagnosis() {
        return icd10diagnosis;
    }

    public List<OWLClassDescription> getTherapies() {
        return therapies;
    }

    public List<OWLClassDescription> getCourses() {
        return courses;
    }

    public String getTumorType() {
        return tumorType;
    }

    public void setTumorType(String tumorType) {
        this.tumorType = tumorType;
    }

    public String getCancerType() {
        return cancerType;
    }

    public void setCancerType(String cancerType) {
        this.cancerType = cancerType;
    }

    public String getOrgan() {
        return organ;
    }

    public void setOrgan(String organ) {
        this.organ = organ;
    }

    public String getIcd10() {
        return icd10;
    }

    public void setIcd10(String icd10) {
        this.icd10 = icd10;
    }

    public String getTherapy() {
        return therapy;
    }

    public void setTherapy(String therapy) {
        this.therapy = therapy;
    }

    public void recommend() {
        actualMonth = 1;
        therapyTypes = new ArrayList<>();
        if (recommendationMonths == null || (recommendationMonths.isEmpty())) {
            JSFUtil.addError("You must define the number of months of the recommendation");
            return;
        }
        Integer m = null;
        try {
            m = Integer.parseInt(recommendationMonths);
        } catch (Exception ex) {
            JSFUtil.addError("The field months must be a positive integer number between 1 and 36");
            return;
        }
        if (m < 1 || m > 36) {
            JSFUtil.addError("The field months must be a positive integer number between 1 and 36");
            return;
        }
        months = m.intValue();
        List<String> filters = new ArrayList<>();
        if (cancerType == null || cancerType.trim().equals("")) {
            JSFUtil.addError("You must define the cancer type");
            return;
        }
        String ct = Util.generateFilterOWLTwoObjectProperty(classToOWL, cancerType.trim(), hasDiagnosis, "ct", hasCancerType, false);
        filters.add(ct);
        if (age == null || age.trim().equals("")) {
            JSFUtil.addError("You must define the patient age");
            return;
        }
        Integer a = null;
        try {
            a = Integer.parseInt(age);
        } catch (Exception ex) {
            JSFUtil.addError("The field age must be a positive integer number between 0 and 120");
            return;
        }
        if (a < 0 || a > 120) {
            JSFUtil.addError("The field age must be a positive integer number between 0 and 120");
            return;
        }
        Integer age_range = Util.getPatientRange();
        Integer min = a - age_range;
        Integer max = a + age_range;
        String ages = "{?s <" + hasAge.toStringID() + "> ?maxAge . FILTER (?maxAge ";
        ages += " >= " + min;
        ages += " && ?maxAge <= " + max;
        ages += " )}";
        filters.add(ages);
        if (gender != null && !gender.trim().equals("")) {
            String g = "{?s <" + hasGender.toStringID() + "> ?gender . FILTER (?gender = '" + gender.trim() + "'^^<http://www.w3.org/2001/XMLSchema#string>)}";
            filters.add(g);
        }
        if (icd10 != null && !icd10.trim().equals("")) {
            String icd = Util.generateFilterOWLTwoObjectProperty(icd10ToOWL, icd10.trim(), hasDiagnosis, "idc", hasICD10_Classification, true);
            filters.add(icd);
        }
        if (tumorType != null && !tumorType.trim().equals("")) {
            String tt = Util.generateFilterOWLTwoObjectProperty(classToOWL, tumorType.trim(), hasDiagnosis, "tt", hasTumorType, false);
            filters.add(tt);
        }
        if (organ != null && !organ.trim().equals("")) {
            String o = Util.generateFilterOWLTwoObjectProperty(classToOWL, organ.trim(), hasDiagnosis, "organ", hasOrgan, false);
            filters.add(o);
        }
        if (therapy != null && !therapy.trim().equals("")) {
            String t = Util.generateFilterOWLWithDiagnosis(classToOWL, therapy.trim(), hasTherapy, "therapy");
            filters.add(t);
        }
        List<OWLClass> ocs = new ArrayList<>();
        ocs.add(classToOWL.getClass("Patient"));
        List<OGIFilter> ofs = new ArrayList<>();
        for (String f : filters) {
            ofs.add(new OGIFilter(f, false));
        }
        List<URI> patients = rdfQuery.findURIByClass(ocs, ofs);
        resultPatientsRecommendation = patients.size();
        pbs = new ArrayList<>();
        for (URI u : patients) {
            PatientBean pb = new PatientBean(u, "Patient");
            pbs.add(pb);
        }
        therapies = new ArrayList<>();
        courses = new ArrayList<>();
        therapiesBeanChart = new ArrayList<>();
        coursesBeanChart = new ArrayList<>();
        for (int i = 0; i < months; i++) {
            HashMap<String, Integer> map = new HashMap<>();
            HashMap<String, Integer> evolutionMap = new HashMap<>();
            int numPatientsTherapy = 0;
            int numPatientsCourse = 0;
            for (PatientBean pb : pbs) {
                for (TherapyBean tb : pb.getTherapies()) {
                    String type = tb.getType();
                    if (i < tb.getArray().size()) {
                        String s = tb.getArray().get(i);
                        if (!s.equalsIgnoreCase("#FFFFFF")) {
                            if (map.containsKey(type)) {
                                numPatientsTherapy++;
                                map.put(type, map.get(type) + 1);
                            } else {
                                map.put(type, 1);
                                numPatientsTherapy++;
                            }
                        }
                    }
                }
                if (pb.getCoursesTable() != null) {
                    if (i < pb.getCoursesTable().size()) {
                        for (CourseBean cb : pb.getCoursesTable().get(i)) {
                            if (evolutionMap.containsKey(cb.getType())) {
                                evolutionMap.put(cb.getType(), evolutionMap.get(cb.getType()) + 1);
                                numPatientsCourse++;
                            } else {
                                evolutionMap.put(cb.getType(), 1);
                                numPatientsCourse++;
                            }
                        }
                    }
                }
            }
            if (!map.isEmpty()) {
                PieChartModelBean pcmb = new PieChartModelBean(map, numPatientsTherapy, "therapies", i + 1);
                therapiesBeanChart.add(pcmb);
            }
            if (!evolutionMap.isEmpty()) {
                PieChartModelBean pcmb = new PieChartModelBean(evolutionMap, numPatientsCourse, "courses", i + 1);
                coursesBeanChart.add(pcmb);
            }
        }
        JSFUtil.addInfo("OK");
    }

    public int getMonths() {
        return months;
    }

    public Integer getResultPatientsRecommendation() {
        return resultPatientsRecommendation;
    }

    public List<PieChartModelBean> getTherapiesBeanChart() {
        return therapiesBeanChart;
    }

    public List<PieChartModelBean> getCoursesBeanChart() {
        return coursesBeanChart;
    }

    public void updateRecommendation() {
        if (therapiesBeanChart == null || therapiesBeanChart.isEmpty()) {
            return;
        }
        if (therapyTypes == null) {
            PieChartModelBean pcmb = therapiesBeanChart.get(0);
            if (pcmb.getTypeSelected() == null || pcmb.getTypeSelected().trim().equals("")) {
                return;
            }
            therapyTypes = new ArrayList<>();
            therapyTypes.add(pcmb.getTypeSelected().trim());
        } else {
            if (therapiesBeanChart.size() > actualMonth - 1) {
                PieChartModelBean pcmb = therapiesBeanChart.get(actualMonth - 1);
                if (pcmb.getTypeSelected() == null || pcmb.getTypeSelected().trim().equals("")) {
                    return;
                }
                therapyTypes.add(pcmb.getTypeSelected().trim());
            }
        }
        String typeFilter = therapyTypes.get(actualMonth - 1);
        List<PatientBean> aux = pbs;
        pbs = new ArrayList<>();
        for (PatientBean pb : aux) {
            boolean finded = false;
            for (TherapyBean tb : pb.getTherapies()) {
                String type = tb.getType();
                if (actualMonth - 1 < tb.getArray().size()) {
                    String s = tb.getArray().get(actualMonth - 1);
                    if (!s.equalsIgnoreCase("#FFFFFF")) {
                        if (type.equals(typeFilter)) {
                            finded = true;
                            break;
                        }
                    }
                }
            }
            if (finded) {
                pbs.add(pb);
            }
        }
        therapiesBeanChart = new ArrayList<>();
        coursesBeanChart = new ArrayList<>();
        actualMonth++;
        resultPatientsRecommendation = pbs.size();
        for (int i = 0; i < months; i++) {
            HashMap<String, Integer> map = new HashMap<>();
            HashMap<String, Integer> evolutionMap = new HashMap<>();
            int numPatientsTherapy = 0;
            int numPatientsCourse = 0;
            for (PatientBean pb : pbs) {
                for (TherapyBean tb : pb.getTherapies()) {
                    String type = tb.getType();
                    if (i < tb.getArray().size()) {
                        String s = tb.getArray().get(i);
                        if (!s.equalsIgnoreCase("#FFFFFF")) {
                            if (map.containsKey(type)) {
                                numPatientsTherapy++;
                                map.put(type, map.get(type) + 1);
                            } else {
                                map.put(type, 1);
                                numPatientsTherapy++;
                            }
                        }
                    }
                }
                if (pb.getCoursesTable() != null) {
                    if (i < pb.getCoursesTable().size()) {
                        for (CourseBean cb : pb.getCoursesTable().get(i)) {
                            if (evolutionMap.containsKey(cb.getType())) {
                                evolutionMap.put(cb.getType(), evolutionMap.get(cb.getType()) + 1);
                                numPatientsCourse++;
                            } else {
                                evolutionMap.put(cb.getType(), 1);
                                numPatientsCourse++;
                            }
                        }
                    }
                }
            }
            if (!map.isEmpty()) {
                if (therapyTypes != null && therapyTypes.size() > i) {
                    PieChartModelBean pcmb = new PieChartModelBean(map, numPatientsTherapy, "therapies", i + 1, therapyTypes.get(i));
                    therapiesBeanChart.add(pcmb);
                } else {
                    PieChartModelBean pcmb = new PieChartModelBean(map, numPatientsTherapy, "therapies", i + 1);
                    therapiesBeanChart.add(pcmb);
                }
            }
            if (!evolutionMap.isEmpty()) {
                PieChartModelBean pcmb = new PieChartModelBean(evolutionMap, numPatientsCourse, "courses", i + 1);
                coursesBeanChart.add(pcmb);
            }
        }
    }
}
