package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.cccf.web.application.OGIFilter;
import es.imib.export.ClassToOWL;
import es.imib.jsf.JSFUtil;
import es.imib.semantic.RDFQuery;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLClass;

@ViewScoped
@ManagedBean
public class RecommendOGIBean extends OntologyGuidedInputWeb {

    protected int actualMonth;
    protected List<String> therapyTypes;
    protected String recommendationMonths;
    protected List<PieChartModelBean> therapiesBeanChart;
    protected List<PieChartModelBean> coursesBeanChart;
    protected List<PatientBean> pbs;
    protected Integer resultPatientsRecommendation;
    protected List<TherapyBean> therapiesBean;
    protected List<CourseBean> coursesBean;
    protected int months;
    protected ClassToOWL classToOWL;
    protected RDFQuery rdfQuery;    

    public RecommendOGIBean() {
        super("Patient");
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        rdfQuery = new RDFQuery(Util.getRepositoryRDF().getRepository());
        therapiesBeanChart = new ArrayList<>();
        coursesBeanChart = new ArrayList<>();
        months = 0;
        actualMonth = 1;        
    }

    @Override
    protected void generateList() {

    }

    public void recommend() {
        actualMonth = 1;
        therapyTypes = new ArrayList<>();
        if (recommendationMonths == null || (recommendationMonths.isEmpty())) {
            JSFUtil.addError("You must define the number of months of the recommendation");
            return;
        }
        Integer m = null;
        try {
            m = Integer.parseInt(recommendationMonths);
        } catch (Exception ex) {
            JSFUtil.addError("The field months must be a positive integer number between 1 and 36");
            return;
        }
        if (m < 1 || m > 36) {
            JSFUtil.addError("The field months must be a positive integer number between 1 and 36");
            return;
        }
        months = m.intValue();
        List<OWLClass> ocs = new ArrayList<>();
        ocs.add(classToOWL.getClass("Patient"));
        List<OGIFilter> ofs = generateFilters();
        List<URI> patients = rdfQuery.findURIByClass(ocs, ofs);
        resultPatientsRecommendation = patients.size();
        pbs = new ArrayList<>();
        for (URI u : patients) {
            PatientBean pb = new PatientBean(u, "Patient");
            pbs.add(pb);
        }
        therapiesBeanChart = new ArrayList<>();
        coursesBeanChart = new ArrayList<>();
        for (int i = 0; i < months; i++) {
            HashMap<String, Integer> map = new HashMap<>();
            HashMap<String, Integer> evolutionMap = new HashMap<>();
            int numPatientsTherapy = 0;
            int numPatientsCourse = 0;
            for (PatientBean pb : pbs) {
                for (TherapyBean tb : pb.getTherapies()) {
                    String type = tb.getType();
                    if (i < tb.getArray().size()) {
                        String s = tb.getArray().get(i);
                        if (!s.equalsIgnoreCase("#FFFFFF")) {
                            if (map.containsKey(type)) {
                                numPatientsTherapy++;
                                map.put(type, map.get(type) + 1);
                            } else {
                                map.put(type, 1);
                                numPatientsTherapy++;
                            }
                        }
                    }
                }
                if (pb.getCoursesTable() != null) {
                    if (i < pb.getCoursesTable().size()) {
                        for (CourseBean cb : pb.getCoursesTable().get(i)) {
                            if (evolutionMap.containsKey(cb.getType())) {
                                evolutionMap.put(cb.getType(), evolutionMap.get(cb.getType()) + 1);
                                numPatientsCourse++;
                            } else {
                                evolutionMap.put(cb.getType(), 1);
                                numPatientsCourse++;
                            }
                        }
                    }
                }
            }
            if (!map.isEmpty()) {                
                PieChartModelBean pcmb = new PieChartModelBean(map, numPatientsTherapy, "therapies", i + 1);
                therapiesBeanChart.add(pcmb);
            }
            if (!evolutionMap.isEmpty()) {                
                PieChartModelBean pcmb = new PieChartModelBean(evolutionMap, numPatientsCourse, "courses", i + 1);
                coursesBeanChart.add(pcmb);
            }
        }
        JSFUtil.addInfo("OK");
    }

    public int getMonths() {
        return months;
    }

    public Integer getResultPatientsRecommendation() {
        return resultPatientsRecommendation;
    }

    public List<PieChartModelBean> getTherapiesBeanChart() {
        return therapiesBeanChart;
    }

    public List<PieChartModelBean> getCoursesBeanChart() {
        return coursesBeanChart;
    }

    public void setRecommendationMonths(String recommendationMonths) {
        this.recommendationMonths = recommendationMonths;
    }

    public void updateRecommendation() {
        if (therapiesBeanChart == null || therapiesBeanChart.isEmpty()) {
            return;
        }
        if (therapyTypes == null) {
            PieChartModelBean pcmb = therapiesBeanChart.get(0);
            if (pcmb.getTypeSelected() == null || pcmb.getTypeSelected().trim().equals("")) {
                return;
            }
            therapyTypes = new ArrayList<>();
            therapyTypes.add(pcmb.getTypeSelected().trim());
        } else {
            if (therapiesBeanChart.size() > actualMonth - 1) {
                PieChartModelBean pcmb = therapiesBeanChart.get(actualMonth - 1);
                if (pcmb.getTypeSelected() == null || pcmb.getTypeSelected().trim().equals("")) {
                    return;
                }
                therapyTypes.add(pcmb.getTypeSelected().trim());
            }
        }
        String typeFilter = therapyTypes.get(actualMonth - 1);
        List<PatientBean> aux = pbs;
        pbs = new ArrayList<>();
        for (PatientBean pb : aux) {
            boolean finded = false;
            for (TherapyBean tb : pb.getTherapies()) {
                String type = tb.getType();
                if (actualMonth - 1 < tb.getArray().size()) {
                    String s = tb.getArray().get(actualMonth - 1);
                    if (!s.equalsIgnoreCase("#FFFFFF")) {
                        if (type.equals(typeFilter)) {
                            finded = true;
                            break;
                        }
                    }
                }
            }
            if (finded) {
                pbs.add(pb);
            }
        }
        therapiesBeanChart = new ArrayList<>();
        coursesBeanChart = new ArrayList<>();
        actualMonth++;
        resultPatientsRecommendation = pbs.size();
        for (int i = 0; i < months; i++) {
            HashMap<String, Integer> map = new HashMap<>();
            HashMap<String, Integer> evolutionMap = new HashMap<>();
            int numPatientsTherapy = 0;
            int numPatientsCourse = 0;
            for (PatientBean pb : pbs) {
                for (TherapyBean tb : pb.getTherapies()) {
                    String type = tb.getType();
                    if (i < tb.getArray().size()) {
                        String s = tb.getArray().get(i);
                        if (!s.equalsIgnoreCase("#FFFFFF")) {
                            if (map.containsKey(type)) {
                                numPatientsTherapy++;
                                map.put(type, map.get(type) + 1);
                            } else {
                                map.put(type, 1);
                                numPatientsTherapy++;
                            }
                        }
                    }
                }
                if (pb.getCoursesTable() != null) {
                    if (i < pb.getCoursesTable().size()) {
                        for (CourseBean cb : pb.getCoursesTable().get(i)) {
                            if (evolutionMap.containsKey(cb.getType())) {
                                evolutionMap.put(cb.getType(), evolutionMap.get(cb.getType()) + 1);
                                numPatientsCourse++;
                            } else {
                                evolutionMap.put(cb.getType(), 1);
                                numPatientsCourse++;
                            }
                        }
                    }
                }
            }
            if (!map.isEmpty()) {                
                if (therapyTypes != null && therapyTypes.size() > i) {
                    PieChartModelBean pcmb = new PieChartModelBean(map, numPatientsTherapy, "therapies", i + 1, therapyTypes.get(i));
                    therapiesBeanChart.add(pcmb);
                } else {
                    PieChartModelBean pcmb = new PieChartModelBean(map, numPatientsTherapy, "therapies", i + 1);
                    therapiesBeanChart.add(pcmb);
                }
            }
            
            if (!evolutionMap.isEmpty()) {
                PieChartModelBean pcmb = new PieChartModelBean(evolutionMap, numPatientsCourse, "courses", i + 1);
                coursesBeanChart.add(pcmb);
            }
        }
    }

    public int getActualMonth() {
        return actualMonth;
    }

    public List<String> getTherapyTypes() {
        return therapyTypes;
    }

    public String getRecommendationMonths() {
        return recommendationMonths;
    }

    public List<PatientBean> getPbs() {
        return pbs;
    }

    public List<TherapyBean> getTherapiesBean() {
        return therapiesBean;
    }

    public List<CourseBean> getCoursesBean() {
        return coursesBean;
    }

}
