package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.export.ClassToOWL;
import es.imib.semantic.RDFQuery;
import es.imib.semantic.Tuple;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import org.openrdf.model.URI;
import org.openrdf.repository.Repository;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;

public abstract class SemanticBean implements Serializable {
    protected String id;
    protected OWLDataProperty idProperty;
    protected URI uri;
    protected ClassToOWL classToOWL;
    protected ClassToOWL icd10ToOWL;
    protected Repository repository;    
    protected RDFQuery rdfQuery;
    protected List<OWLClass> domain;
    protected List<OWLClass> types;
    
    public SemanticBean() {
        
    }

    public SemanticBean(URI uri, String type) {
        this.uri = uri;
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        icd10ToOWL = Util.getOntologyOWL().getIcd10ToOWL();
        repository = Util.getRepositoryRDF().getRepository();
        idProperty = classToOWL.getDataProperty("id");
        rdfQuery = new RDFQuery(repository);
        domain = loadDomain(type);
    }

    public SemanticBean(String id, String type) {
        this.id = id;
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        repository = Util.getRepositoryRDF().getRepository();
        idProperty = classToOWL.getDataProperty("id");
        rdfQuery = new RDFQuery(repository);
        domain = loadDomain(type);
    }
    
    protected List<String> getListOWLClass() {
        List<String> ocs = new ArrayList<>();
        for (OWLClass oc : domain) {
            ocs.add(oc.toStringID());
        }
        return ocs;
    }
    
    protected List<OWLClass> loadDomain(String type) {
        OWLClass oc = classToOWL.getClass(type);
        List<OWLClass> ocs = classToOWL.getAllSubclasses(oc);
        ocs.add(oc);
        return ocs;
    }
    
    protected void loadTypes(List<Tuple> tuples) {
        types = new ArrayList<>();
        for (Tuple t : tuples) {
            if (t.getPredicate().stringValue().equals("http://www.w3.org/1999/02/22-rdf-syntax-ns#type")) {
                for (OWLClass oc : domain) {                    
                    if (t.getObject().stringValue().equals(oc.toStringID())) {
                        types.add(oc);
                    }
                }
            }
        }
    }

    public List<OWLClass> getDomain() {
        return domain;
    }

    public void setDomain(List<OWLClass> domain) {
        this.domain = domain;
    }

    public List<OWLClass> getTypes() {
        return types;
    }

    public void setTypes(List<OWLClass> types) {
        this.types = types;
    }

    public RDFQuery getRdfQuery() {
        return rdfQuery;
    }

    public String getId() {
        return id;
    }

    public OWLDataProperty getIdProperty() {
        return idProperty;
    }

    public URI getUri() {
        return uri;
    }

    public ClassToOWL getClassToOWL() {
        return classToOWL;
    }

    public Repository getRepository() {
        return repository;
    }
}
