package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.semantic.Tuple;
import java.util.Date;
import java.util.List;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public abstract class SemanticDateBean extends SemanticBean {
    // Annotation properties
    public static String ID = "id";
    // Data properties
    public static String CREATIONDATE = "creationDate";
    public static String EDITIONDATE = "editionDate";
    // Object properties
    public static String CREATIONCLINICALDEPARTMENT = "hasCreationClinicalDepartment";
    public static String EDITIONCLINICALDEPARTMENT = "hasEditionClinicalDepartment";
    // Properties
    protected Date creationDate;
    protected Date editionDate;
    protected DepartmentBean creationDepartment;
    protected DepartmentBean editionDepartment;
    protected PatientBean patient;
    
    public SemanticDateBean() {
        
    }

    public SemanticDateBean(PatientBean patient, URI uri, String type) {
        super(uri, type);
        this.patient = patient;
    }
    
    protected void loadSemanticInformation(List<Tuple> tuples) {
        OWLDataProperty creationDateProperty = classToOWL.getDataProperty(CREATIONDATE);
        OWLDataProperty editionDateProperty = classToOWL.getDataProperty(EDITIONDATE);
        OWLObjectProperty hasCreationClinicalDepartment = classToOWL.getObjetProperty(CREATIONCLINICALDEPARTMENT);
        OWLObjectProperty hasEditionClinicalDepartment = classToOWL.getObjetProperty(EDITIONCLINICALDEPARTMENT);
        OWLAnnotationProperty idProperty = classToOWL.getAnnotationProperty(ID);         
        loadTypes(tuples);
        for (Tuple t : tuples) {
            if (t.getPredicate().stringValue().equals(creationDateProperty.toStringID())) {
                String s = t.getObject().stringValue();
                creationDate = Util.convertXMLDate(s);
            } else if (t.getPredicate().stringValue().equals(editionDateProperty.toStringID())) {
                String s = t.getObject().stringValue();
                editionDate = Util.convertXMLDate(s);
            } else if (t.getPredicate().stringValue().equals(hasCreationClinicalDepartment.toStringID())) {
                URI u = (URI) t.getObject();
                creationDepartment = new DepartmentBean(u, "ClinicalDepartment");
            } else if (t.getPredicate().stringValue().equals(hasEditionClinicalDepartment.toStringID())) {
                URI u = (URI) t.getObject();
                editionDepartment = new DepartmentBean(u, "ClinicalDepartment");
            } else if (t.getPredicate().stringValue().equals(idProperty.toStringID())) {
                id = t.getObject().stringValue();
            } else {
                loadOtherSemanticInformation(t);
            }
        }
    }
    
    protected abstract void loadOtherSemanticInformation(Tuple t);

    public Date getCreationDate() {
        return creationDate;
    }

    public Date getEditionDate() {
        return editionDate;
    }

    public DepartmentBean getCreationDepartment() {
        return creationDepartment;
    }

    public DepartmentBean getEditionDepartment() {
        return editionDepartment;
    }

    public PatientBean getPatient() {
        return patient;
    }
}
