package es.imib.cccf.semantic.beans;

import es.imib.cccf.util.Util;
import es.imib.semantic.Tuple;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLDataProperty;

public class TherapyBean extends SemanticDateBean implements Comparable<TherapyBean> {
    // Data properties 
    public static String STARTDATE = "startDate";
    public static String ENDDATE = "endDate";
    // Properties
    protected Date startDate;
    protected Date endDate;
    protected DiagnosisBean diagnosis;
    protected OWLDataProperty startDateProperty;
    protected OWLDataProperty endDateProperty;
    protected int months;
    protected List<String> array;

    public TherapyBean() {
    }

    public TherapyBean(PatientBean patient, URI uri, String type, DiagnosisBean diagnosis) {
        super(patient, uri, type);
        this.diagnosis = diagnosis;
        startDateProperty = classToOWL.getDataProperty(STARTDATE);
        endDateProperty = classToOWL.getDataProperty(ENDDATE);        
        List<Tuple> tuples = rdfQuery.runQueryTuplesByURI(uri);
        loadSemanticInformation(tuples);        
        if (endDate == null) {
            months = 1;
        } else {
            Calendar calendarStart = Calendar.getInstance();
            calendarStart.setTime(startDate);
            Calendar calendarEnd = Calendar.getInstance();
            calendarEnd.setTime(endDate);
            while (calendarStart.get(Calendar.MONTH) != calendarEnd.get(Calendar.MONTH)
                    || calendarStart.get(Calendar.YEAR) != calendarEnd.get(Calendar.YEAR)) {
                months++;
                calendarStart.add(Calendar.MONTH, 1);
            }
            months++;
        }        
    }
    
    public String getType() {
        if (types == null) {
            return "";
        }
        if (types.isEmpty()) {
            return "";
        }
        return types.get(0).getIRI().getFragment();
    }

    public List<String> getArray() {
        return array;
    }

    public Date getStartDate() {
        return startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    @Override
    protected void loadOtherSemanticInformation(Tuple t) {
        if (t.getPredicate().stringValue().equals(startDateProperty.toStringID())) {
            String s = t.getObject().stringValue();
            startDate = Util.convertXMLDate(s);
        } else if (t.getPredicate().stringValue().equals(endDateProperty.toStringID())) {
            String s = t.getObject().stringValue();
            endDate = Util.convertXMLDate(s);
        }
    }

    @Override
    public int compareTo(TherapyBean o) {
        return this.startDate.compareTo(o.getStartDate());
    }

    public DiagnosisBean getDiagnosis() {
        return diagnosis;
    }
    
    public void loadTimes() {
        array = new ArrayList<>();
        Calendar patientDate = Calendar.getInstance();
        patientDate.setTime(patient.getStartDate());
        Calendar therapyDate = Calendar.getInstance();
        therapyDate.setTime(startDate);
        boolean finded  = false;
        for (int i = 0; i < patient.getMonths(); i++) {            
            if (therapyDate.get(Calendar.YEAR) == patientDate.get(Calendar.YEAR)) {
                if (therapyDate.get(Calendar.MONTH) == patientDate.get(Calendar.MONTH)) {
                    for (int j = 0; j < months; j++, i++) {
                        array.add("#FF3F3F");
                    }
                    for (; i < patient.getMonths(); i++) {
                        array.add("#FFFFFF");
                    }
                    finded = true;
                    break;
                }
            }
            array.add("#FFFFFF");
            patientDate.add(Calendar.MONTH, 1);
        }
        if (!finded) {
            array.remove(array.size() - 1);
            array.add("#FF3F3F");
        }
        
    }
}
