package es.imib.cccf.semantic.collections;

import es.imib.cccf.beans.PrimefacesSemanticList;
import es.imib.cccf.semantic.collections.beans.PatientBean;
import es.imib.cccf.util.Util;
import es.imib.cccf.web.application.OGIFilter;
import es.imib.semantic.Tuple;
import java.util.ArrayList;
import java.util.List;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class PatientList extends PrimefacesSemanticList<PatientBean> {
    protected List<OWLClassDescription> tumorTypes;
    protected List<OWLClassDescription> cancerTypes;
    protected List<OWLClassDescription> organs;
    protected List<OWLClassDescription> icd10diagnosis;
    protected List<OWLClassDescription> therapies;
    protected List<OWLClassDescription> courses;    
    protected OWLObjectProperty hasDiagnosis;
    protected OWLObjectProperty hasTherapy;
    protected OWLObjectProperty hasDiseaseCourse;
    protected OWLObjectProperty hasICD10_Classification;
    protected OWLObjectProperty hasTumorType;
    protected OWLObjectProperty hasCancerType;
    protected OWLObjectProperty hasOrgan;
    protected OWLDataProperty hasAge;
    protected OWLDataProperty hasGender;
    protected OWLAnnotationProperty hasId;    
    protected String gender;
    protected String genderSearch;
    protected String minAge;
    protected String maxAge;
    protected String minAgeSearch;
    protected String maxAgeSearch;
    protected String tumorType;
    protected String tumorTypeSearch;
    protected String cancerType;
    protected String cancerTypeSearch;
    protected String organ;
    protected String organSearch;
    protected String icd10;
    protected String icd10Search;
    protected String id;
    protected String idSearch;
    protected String therapy;
    protected String therapySearch;
    protected String course;
    protected String courseSearch;
    
    public PatientList() {
        super("Patient");
        OWLAnnotationProperty owlap = classToOWL.getAnnotationProperty("http://www.w3.org/2000/01/rdf-schema", "label");
        OWLAnnotationProperty owlde = icd10ToOWL.getAnnotationProperty("has_Description");
        tumorTypes = Util.getListOWLClassDescription("TumorType", classToOWL, false, owlap);        
        cancerTypes = Util.getListOWLClassDescription("CancerType", classToOWL, false, owlap);        
        organs = Util.getListOWLClassDescription("Organ", classToOWL, false, owlap);
        icd10diagnosis = Util.getListOWLClassDescription("C00_D48", icd10ToOWL, true, owlde);
        //icd10diagnosis.addAll(Util.getListOWLClassDescription("", icd10ToOWL, true, owlde));
        therapies = Util.getListOWLClassDescription("Therapy", classToOWL, true, owlap);
        courses = Util.getListOWLClassDescription("DiseaseCourse", classToOWL, true, owlap);
        hasDiagnosis = classToOWL.getObjetProperty("hasDiagnosis");
        hasDiseaseCourse = classToOWL.getObjetProperty("hasDiseaseCourse");
        hasTherapy = classToOWL.getObjetProperty("hasTherapy");
        hasCancerType = classToOWL.getObjetProperty("hasPathologicalStructure");
        hasTumorType = classToOWL.getObjetProperty("hasTumorType");
        hasOrgan = classToOWL.getObjetProperty("hasAnatomicalStructure");
        hasICD10_Classification = classToOWL.getObjetProperty("hasICD10_Classification");
        hasAge = classToOWL.getDataProperty("age");
        hasGender = classToOWL.getDataProperty("gender");       
        hasId = classToOWL.getAnnotationProperty("id");
    }

    public String getGenderSearch() {
        return genderSearch;
    }

    public void setGenderSearch(String genderSearch) {
        this.genderSearch = genderSearch;
    }

    public String getMinAgeSearch() {
        return minAgeSearch;
    }

    public void setMinAgeSearch(String minAgeSearch) {
        this.minAgeSearch = minAgeSearch;
    }

    public String getMaxAgeSearch() {
        return maxAgeSearch;
    }

    public void setMaxAgeSearch(String maxAgeSearch) {
        this.maxAgeSearch = maxAgeSearch;
    }

    public String getTumorTypeSearch() {
        return tumorTypeSearch;
    }

    public void setTumorTypeSearch(String tumorTypeSearch) {
        this.tumorTypeSearch = tumorTypeSearch;
    }

    public String getCancerTypeSearch() {
        return cancerTypeSearch;
    }

    public void setCancerTypeSearch(String cancerTypeSearch) {
        this.cancerTypeSearch = cancerTypeSearch;
    }

    public String getOrganSearch() {
        return organSearch;
    }

    public void setOrganSearch(String organSearch) {
        this.organSearch = organSearch;
    }

    public String getIcd10Search() {
        return icd10Search;
    }

    public void setIcd10Search(String icd10Search) {
        this.icd10Search = icd10Search;
    }

    public String getIdSearch() {
        return idSearch;
    }

    public void setIdSearch(String idSearch) {
        this.idSearch = idSearch;
    }

    public String getTherapySearch() {
        return therapySearch;
    }

    public void setTherapySearch(String therapySearch) {
        this.therapySearch = therapySearch;
    }

    public String getCourseSearch() {
        return courseSearch;
    }

    public void setCourseSearch(String courseSearch) {
        this.courseSearch = courseSearch;
    }

    public List<OWLClassDescription> getTumorTypes() {
        return tumorTypes;
    }

    public List<OWLClassDescription> getCancerTypes() {
        return cancerTypes;
    }

    public List<OWLClassDescription> getOrgans() {
        return organs;
    }

    public List<OWLClassDescription> getIcd10diagnosis() {
        return icd10diagnosis;
    }

    public List<OWLClassDescription> getTherapies() {
        return therapies;
    }

    public List<OWLClassDescription> getCourses() {
        return courses;
    }

    @Override
    public void buscar() {
        filters = new ArrayList<>();
        gender = null;
        if (genderSearch != null && !genderSearch.trim().equals("")) {
            gender = "{?s <" + hasGender.toStringID() + "> ?gender . FILTER (?gender = '" + genderSearch.trim() + "'^^<http://www.w3.org/2001/XMLSchema#string>)}";
            OGIFilter of = new OGIFilter(gender, false);
            filters.add(of);
        }
        minAge = null;
        Integer min = null;
        if (minAgeSearch != null && !minAgeSearch.trim().equals("")) {
            try {
                min = Integer.parseInt(minAgeSearch.trim());                                
            } catch (Exception ex) {
                
            }
        }
        maxAge = null;
        Integer max = null;
        if (maxAgeSearch != null && !maxAgeSearch.trim().equals("")) {
            try {
                max = Integer.parseInt(maxAgeSearch.trim());                
            } catch (Exception ex) {
                
            }
        }
        if (max != null || min != null) {
            maxAge = "{?s <" + hasAge.toStringID() + "> ?maxAge . FILTER (?maxAge ";
            if (min != null) {
                maxAge += " >= " + min;
                if (max != null) {
                    maxAge += " && ?maxAge <= " + max;
                }
            } else {
                maxAge += " <= " + max;
            }
            maxAge += " )}";
            OGIFilter of = new OGIFilter(maxAge, false);
            filters.add(of);
        }
        id = null;
        if (idSearch != null && !idSearch.trim().equals("")) {
            id = "{?s <" + hasId.toStringID() + "> ?id . FILTER (?id = '" + idSearch.trim() + "'^^<http://www.w3.org/2001/XMLSchema#string>)}";
            OGIFilter of = new OGIFilter(id, false);
            filters.add(of);
        }
        course = null;
        if (courseSearch != null && !courseSearch.trim().equals("")) {            
            course = Util.generateFilterOWLWithDiagnosis(classToOWL, courseSearch.trim(), hasDiseaseCourse, "course");
            OGIFilter of = new OGIFilter(course, false);
            filters.add(of);
        }
        icd10 = null;
        if (icd10Search != null && !icd10Search.trim().equals("")) {
            icd10 = Util.generateFilterOWLTwoObjectProperty(icd10ToOWL, icd10Search.trim(), hasDiagnosis, "idc", hasICD10_Classification, true);
            OGIFilter of = new OGIFilter(icd10, false);
            filters.add(of);
        }
        therapy = null;
        if (therapySearch != null && !therapySearch.trim().equals("")) {
            therapy = Util.generateFilterOWLWithDiagnosis(classToOWL, therapySearch.trim(), hasTherapy, "therapy");
            OGIFilter of = new OGIFilter(therapy, false);
            filters.add(of);
        }
        organ = null;
        if (organSearch != null && !organSearch.trim().equals("")) {
            organ = Util.generateFilterOWLTwoObjectProperty(classToOWL, organSearch.trim(), hasDiagnosis, "organ", hasOrgan, false);
            OGIFilter of = new OGIFilter(organ, false);
            filters.add(of);
        }
        tumorType = null;
        if (tumorTypeSearch != null && !tumorTypeSearch.trim().equals("")) {
            tumorType = Util.generateFilterOWLTwoObjectProperty(classToOWL, tumorTypeSearch.trim(), hasDiagnosis, "tt", hasTumorType, false);
            OGIFilter of = new OGIFilter(tumorType, false);
            filters.add(of);
        }
        cancerType = null;
        if (cancerTypeSearch != null && !cancerTypeSearch.trim().equals("")) {
            cancerType = Util.generateFilterOWLTwoObjectProperty(classToOWL, cancerTypeSearch.trim(), hasDiagnosis, "ct", hasCancerType, false);
            OGIFilter of = new OGIFilter(cancerType, false);
            filters.add(of);
        }
        numResults = findTotalResults();
    }

    @Override
    public void listarTodos() {
        gender = null;
        genderSearch = null;
        maxAge = null;
        maxAgeSearch = null;
        minAge = null;
        minAgeSearch = null;
        tumorType = null;
        tumorTypeSearch = null;
        cancerType = null;
        cancerTypeSearch = null;
        organ = null;
        organSearch = null;
        therapy = null;
        therapySearch = null;
        icd10 = null;
        icd10Search = null;
        id = null;
        idSearch = null;
        course = null;
        courseSearch = null;
        filters = new ArrayList<>();
        numResults = findTotalResults();
    }

    @Override
    protected PatientBean createBean(URI uri, List<Tuple> tuples) {
        return new PatientBean(uri, tuples);
    }
}
