package es.imib.cccf.semantic.collections.beans;

import es.imib.cccf.util.Util;
import es.imib.export.ClassToOWL;
import es.imib.semantic.Tuple;
import java.io.Serializable;
import java.util.List;
import org.openrdf.model.URI;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;

public abstract class SemanticBeanCollection implements Serializable {
    protected URI uri;
    protected ClassToOWL classToOWL;
    protected String id;
    protected OWLAnnotationProperty idProperty;

    public SemanticBeanCollection(URI uri, List<Tuple> tuples) {
        this.uri = uri;
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        idProperty = classToOWL.getAnnotationProperty("id");
        if (tuples != null && !tuples.isEmpty()) {
            loadSemanticInformation(tuples);
        }
    }
    
    protected abstract void loadSemanticInformation(List<Tuple> tuples);
    
    protected void loadId(Tuple t) {
        if (t.getPredicate().stringValue().equals(idProperty.toStringID())) {
            id = t.getObject().stringValue();
        }
    }

    public URI getUri() {
        return uri;
    }

    public String getId() {
        return id;
    }
}
