package es.imib.cccf.util;

import com.sun.org.apache.xerces.internal.jaxp.datatype.XMLGregorianCalendarImpl;
import es.imib.cccf.semantic.collections.OWLClassDescription;
import es.imib.cccf.web.application.OGIObjectProperty;
import es.imib.cccf.web.application.OntologyGuidedInputCache;
import es.imib.cccf.web.application.OntologyOWL;
import es.imib.cccf.web.application.PatientRange;
import es.imib.cccf.web.application.RepositoryRDF;
import es.imib.export.ClassToOWL;
import es.imib.jsf.JSFUtil;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import javax.faces.context.FacesContext;
import org.primefaces.model.chart.PieChartModel;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class Util {

    public static OntologyOWL getOntologyOWL() {
        if (JSFUtil.getApplicationBean("ontologyOWL") == null) {
            OntologyOWL ontologyOWL = new OntologyOWL();
            FacesContext.getCurrentInstance().getExternalContext().getApplicationMap().put("ontologyOWL", ontologyOWL);
            return ontologyOWL;
        }
        return (OntologyOWL) JSFUtil.getApplicationBean("ontologyOWL");
    }
    
    public static OntologyGuidedInputCache getOntologyGuidedInputCache() {
        if (JSFUtil.getApplicationBean("ontologyGuidedInputCache") == null) {
            OntologyGuidedInputCache ontologyGuidedInputCache = new OntologyGuidedInputCache();
            FacesContext.getCurrentInstance().getExternalContext().getApplicationMap().put("ontologyGuidedInputCache", ontologyGuidedInputCache);
            return ontologyGuidedInputCache;
        }
        return (OntologyGuidedInputCache) JSFUtil.getApplicationBean("ontologyGuidedInputCache");
    }

    public static RepositoryRDF getRepositoryRDF() {
        if (JSFUtil.getApplicationBean("repositoryRDF") == null) {
            RepositoryRDF repositoryRDF = new RepositoryRDF();
            FacesContext.getCurrentInstance().getExternalContext().getApplicationMap().put("repositoryRDF", repositoryRDF);
            return repositoryRDF;
        }
        return (RepositoryRDF) JSFUtil.getApplicationBean("repositoryRDF");
    }
    
    public static int getPatientRange() {
        if (JSFUtil.getApplicationBean("patientRange") == null) {
            PatientRange pr = new PatientRange();
            FacesContext.getCurrentInstance().getExternalContext().getApplicationMap().put("patientRange", pr);
            return pr.getPatientsRange();
        }
        return ((PatientRange) JSFUtil.getApplicationBean("patientRange")).getPatientsRange();
    }
    
    public static PatientRange getPatientRangeBean() {
        if (JSFUtil.getApplicationBean("patientRange") == null) {
            PatientRange pr = new PatientRange();
            FacesContext.getCurrentInstance().getExternalContext().getApplicationMap().put("patientRange", pr);
            return pr;
        }
        return ((PatientRange) JSFUtil.getApplicationBean("patientRange"));
    }

    public static Date convertXMLDate(String date) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        try {
            return sdf.parse(date);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
    
    public static Date convertXMLDate(String date, String mask) {
        SimpleDateFormat sdf = new SimpleDateFormat(mask);
        try {
            return sdf.parse(date);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
    
    public static String getXMLDate(Date value) {
        GregorianCalendar gc = new GregorianCalendar();
        gc.setTime(value);
        XMLGregorianCalendarImpl xml = new XMLGregorianCalendarImpl(gc);
        return xml.toXMLFormat();
    }

    public static List<OWLClassDescription> getListOWLClassDescription(String fatherClass,
            ClassToOWL classToOWL, boolean recursive, OWLAnnotationProperty property) {
        List<OWLClassDescription> result = new ArrayList<>();
        OWLClass oc = classToOWL.getClass(fatherClass);
        List<OWLClass> ocs;
        if (recursive) {
            ocs = classToOWL.getAllSubclasses(oc);
        } else {
            ocs = classToOWL.getAllSubclassesNoRecursive(oc);
        }
        ocs.add(oc);
        for (OWLClass o : ocs) {
            OWLClassDescription ocd = new OWLClassDescription(o, classToOWL, property);
            result.add(ocd);
        }
        return result;
    }

    public static String generateFilterOWLWithDiagnosis(ClassToOWL classToOWL, String className, OWLObjectProperty objectProperty, String paramName) {        
        OWLClass oc = classToOWL.getClassByURI(className.trim());
        List<OWLClass> ocs = classToOWL.getAllSubclasses(oc);
        String param = " { {?" + paramName + " rdf:type <" + oc.toStringID() + ">} ";
        for (OWLClass o : ocs) {
            param += " UNION {?" + paramName + " rdf:type <" + o.toStringID() + ">} ";
        }
        param += "}";
        String result = "{?s <" + objectProperty.toStringID() + "> ?" + paramName + " . "
                + "{?" + paramName + " <http://www.imib.es/ontologies/disease-times#hasDiagnosis> ?diagnosis" + paramName + "} . " + param + "}";
        return result;
    }
    
    public static String generateFilterOWL(ClassToOWL classToOWL, String className, OWLObjectProperty objectProperty, String paramName) {        
        OWLClass oc = classToOWL.getClassByURI(className.trim());
        List<OWLClass> ocs = classToOWL.getAllSubclasses(oc);
        String param = " { {?" + paramName + " rdf:type <" + oc.toStringID() + ">} ";
        for (OWLClass o : ocs) {
            param += " UNION {?" + paramName + " rdf:type <" + o.toStringID() + ">} ";
        }
        param += "}";
        String result = "{?s <" + objectProperty.toStringID() + "> ?" + paramName + " . "
                + param + "}";
        return result;
    }
    
    public static String generateFilterOWL(String variable, ClassToOWL classToOWL, String className, OGIObjectProperty objectProperty, String paramName) {        
        OWLClass oc = classToOWL.getClassByURI(className.trim());
        List<OWLClass> ocs = classToOWL.getAllSubclasses(oc);
        String param = " { {?" + paramName + " rdf:type <" + oc.toStringID() + ">} ";
        for (OWLClass o : ocs) {
            param += " UNION {?" + paramName + " rdf:type <" + o.toStringID() + ">} ";
        }
        param += "}";
        String result = "{" + variable + " <" + objectProperty.getUri() + "> ?" + paramName + " . "
                + param + "}";
        return result;
    }
    
    public static String generateFilterTypeOWL(String variable, ClassToOWL classToOWL, String className) {        
        OWLClass oc = classToOWL.getClassByURI(className.trim());
        List<OWLClass> ocs = classToOWL.getAllSubclasses(oc);
        String param = " { {" + variable + " rdf:type <" + oc.toStringID() + ">} ";
        for (OWLClass o : ocs) {
            param += " UNION {" + variable + " rdf:type <" + o.toStringID() + ">} ";
        }
        param += "}";        
        return param;
    }
    
    public static String generateFilterOWLTwoObjectProperty(ClassToOWL classToOWL, String className, 
            OWLObjectProperty objectProperty, String paramName,
            OWLObjectProperty objectProperty2, boolean recursive) {        
        OWLClass oc = classToOWL.getClassByURI(className.trim());
        List<OWLClass> ocs;
        if (recursive) {
            ocs = classToOWL.getAllSubclasses(oc);
        } else {
            ocs = new ArrayList<>();
            ocs.add(oc);
        }        
        String param = " { {?" + paramName + " rdf:type <" + oc.toStringID() + ">} ";
        for (OWLClass o : ocs) {
            param += " UNION {?" + paramName + " rdf:type <" + o.toStringID() + ">} ";
        }
        param += "}";
        String result = "{?s <" + objectProperty.toStringID() + "> ?" + paramName + "1 . "
                + "{?" + paramName + "1 <" + objectProperty2.toStringID() + "> ?" + paramName + "} . "
                + param + "}";
        return result;
    }
    
    public static PieChartModel constructPieChartModel(HashMap<String, Integer> map, String title, String seriesColor) {
        PieChartModel pieChartModel = new PieChartModel();
        for (String s : map.keySet()) {
            pieChartModel.set(s, map.get(s));            
        }
        pieChartModel.setShowDataLabels(true);
        pieChartModel.setLegendPosition("s");
        pieChartModel.setDiameter(200);
        if (seriesColor != null) {
            pieChartModel.setSeriesColors(seriesColor);
        }        
        //pieChartModel.setSeriesColors("000000,cdcdcd,efefef,030303,040404,050505,060606,070707,080808,090909,0a0a0a,0b0b0b,0c0c0c,0d0d0d,0e0e0e,0f0f0f");
        pieChartModel.setTitle(title);
        return pieChartModel;
    }
}
