package es.imib.cccf.web.application;

import java.util.ArrayList;
import java.util.List;
import org.semanticweb.owlapi.model.OWLClass;

public class OGIClass extends OGIOwlItem<OWLClass> {
    protected List<OGIDataProperty> dataProperties;
    protected List<OGIObjectProperty> objectProperties;
    protected List<OGIOwlItem> properties;
    protected List<OGIClass> fathers;

    public OGIClass() {
        dataProperties = new ArrayList<>();
        objectProperties = new ArrayList<>();
        fathers = new ArrayList<>();
        properties = new ArrayList<>();
    }

    public OGIClass(OWLClass bean) {
        super(bean);
        dataProperties = new ArrayList<>();
        objectProperties = new ArrayList<>();        
        fathers = new ArrayList<>();
        properties = new ArrayList<>();
    }

    public void addDataProperty(OGIDataProperty dataProperty) {
        for (OGIDataProperty ogidp : dataProperties) {
            if (ogidp.getUri().equals(dataProperty.getUri())) {
                return;
            }
        }
        dataProperties.add(dataProperty);
    }

    public List<OGIOwlItem> getProperties() {
        return properties;
    }
    
    public void addObjectProperty(OGIObjectProperty objectProperty) {
        for (OGIObjectProperty ogiop : objectProperties) {
            if (ogiop.getUri().equals(objectProperty.getUri())) {
                return;
            }
        }
        objectProperties.add(objectProperty);
    }

    public List<OGIDataProperty> getDataProperties() {
        return dataProperties;
    }

    public List<OGIObjectProperty> getObjectProperties() {
        return objectProperties;
    }

    @Override
    public void loadSemanticInformacion(List<OGIClass> ogics) {
        OWLClass oc = (OWLClass) bean;
        List<OWLClass> ocs = ontologyOWL.getClassToOWL().getAllSuperclasses(oc);
        for (OWLClass o : ocs) {
            for (OGIClass ogic : ogics) {
                if (o.toStringID().equals(ogic.getUri())) {
                    addFather(ogic);
                }
            }
        }
        for (OGIClass ogic : fathers) {
            for (OGIDataProperty ogidp : ogic.getDataProperties()) {
                addDataProperty(ogidp);
                addProperty(ogidp);
            }
            for (OGIObjectProperty ogiop : ogic.getObjectProperties()) {
                addObjectProperty(ogiop);
                addProperty(ogiop);
            }
        }
    }
    
    protected void addProperty(OGIOwlItem property) {
        for (OGIOwlItem item : properties) {
            if (item.getUri().equals(property.getUri())) {
                return;
            }
        }
        properties.add(property);
    }
    
    protected void addFather(OGIClass ogic) {
        for (OGIClass oc : fathers) {
            if (oc.getUri().equals(ogic.getUri())) {
                return;
            }
        }
        fathers.add(ogic);
    }
}
