package es.imib.cccf.web.application;

import es.imib.cccf.visitors.OWLClassExpressionVisitorOGI;
import java.util.ArrayList;
import java.util.List;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;

public class OGIObjectProperty extends OGIOwlItem<OWLObjectProperty> {
    public List<OGIClass> sources;
    public List<OGIClass> destinations;

    public OGIObjectProperty(OWLObjectProperty bean) {
        super(bean);
        sources = new ArrayList<>();
        destinations = new ArrayList<>();
    }
    
    @Override
    public void loadSemanticInformacion(List<OGIClass> ogics) {
        for (OWLAxiom oa : bean.getReferencingAxioms(ontologyOWL.getClassToOWL().getOntology(), true)) {            
            if (oa instanceof OWLSubClassOfAxiom) {                
                OWLSubClassOfAxiom axiom = (OWLSubClassOfAxiom) oa;
                if (axiom.getSubClass() instanceof OWLClass) {
                    OWLClass o = (OWLClass) axiom.getSubClass().asOWLClass();                    
                    OWLClassExpressionVisitorOGI expressionVisitorOGI = new OWLClassExpressionVisitorOGI();
                    axiom.getSuperClass().accept(expressionVisitorOGI);
                    for (OGIClass ogic : ogics) {
                        if (ogic.getUri().equals(o.toStringID())) {
                            addSource(ogic);
                            ogic.addObjectProperty(this);
                            ogic.addProperty(this);
                        }
                        if (expressionVisitorOGI.getOc() != null) {
                            if (ogic.getUri().equals(expressionVisitorOGI.getOc().toStringID())) {
                                addDestination(ogic);
                                OWLClass father = ontologyOWL.getClassToOWL().getClassByURI(ogic.getUri());
                                List<OWLClass> children = ontologyOWL.getClassToOWL().getAllSubclasses(father);
                                for (OWLClass ch : children) {
                                    for (OGIClass oaux : ogics) {
                                        if (ch.toStringID().equals(oaux.getUri())) {
                                            addDestination(oaux);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    protected void addSource(OGIClass ogic) {
        for (OGIClass o : sources) {
            if (o.getUri().equals(ogic.getUri())) {
                return;
            }
        }
        sources.add(ogic);
    }
    
    protected void addDestination(OGIClass ogic) {
        for (OGIClass o : destinations) {
            if (o.getUri().equals(ogic.getUri())) {
                return;
            }
        }
        destinations.add(ogic);
    }

    public List<OGIClass> getSources() {
        return sources;
    }

    public List<OGIClass> getDestinations() {
        return destinations;
    }
}
