package es.imib.cccf.web.application;

import es.imib.cccf.util.Util;
import java.io.Serializable;
import java.util.List;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLEntity;
import org.semanticweb.owlapi.vocab.OWLRDFVocabulary;

public abstract class OGIOwlItem<T> implements Serializable {

    protected String label;
    protected String uri;
    protected String fragment;
    protected String color;
    protected OWLEntity bean;
    protected OntologyOWL ontologyOWL;
    protected OWLAnnotationProperty labelAnnotation;

    public OGIOwlItem() {
    }

    public OGIOwlItem(T bean) {
        this.bean = (OWLEntity) bean;
        ontologyOWL = Util.getOntologyOWL();
        uri = this.bean.toStringID();
        fragment = this.bean.getIRI().getFragment();
        labelAnnotation = ontologyOWL.getClassToOWL().getAnnotationProperty(OWLRDFVocabulary.RDFS_LABEL);
        label = ontologyOWL.getClassToOWL().getAnnotationPropertyOwlEntity(labelAnnotation, this.bean);
        if (bean instanceof OWLClass) {
            if (!fragment.equals(label)) {
                label = label + " (" + fragment + ")";
            }
        }
        OWLAnnotationProperty colorAnnotation = ontologyOWL.getClassToOWL().getAnnotationProperty("color");
        color = ontologyOWL.getClassToOWL().getAnnotationPropertyOwlEntity(colorAnnotation, this.bean);
        if (color != null) {
            color = color.replaceAll("#", "");
        }
    }

    public String getColor() {
        return color;
    }

    public String getFragment() {
        return fragment;
    }

    public String getLabel() {
        return label;
    }

    public String getUri() {
        return uri;
    }

    public OWLEntity getBean() {
        return bean;
    }

    public abstract void loadSemanticInformacion(List<OGIClass> ogics);
}
