package es.imib.cccf.web.application;

import es.imib.cccf.util.Util;
import es.imib.export.ClassToOWL;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class OGIStatement implements Serializable {

    protected OGIStatement father;
    protected List<OGIStatement> allStatements;
    protected List<OGIStatement> statements;
    protected List<OGIStatement> children;
    protected List<OGIOwlItem> items;
    protected int number;
    protected String prefix;
    protected String variable;
    protected OGIClass ogiClass;
    protected String propertySelected;
    protected OGIDataProperty dataPropertySelected;
    protected OGIObjectProperty objectPropertySelected;
    protected int variableNum;
    protected int level;
    protected boolean condition;
    protected boolean firstStatement;
    protected String classSelected;
    protected ClassToOWL classToOWL;
    protected Integer operator;
    protected String dataValue;
    protected String dateTimeFunction;
    protected String mask;
    protected boolean datetime;
    protected boolean viewAddStatement;
    protected String variableDestination;

    public OGIStatement(OGIStatement father, List<OGIStatement> statements, String prefix, String variable, OGIClass ogiClass, List<OGIStatement> allStatements, Integer variableNum, int level, boolean firstStatement) {
        this.variable = variable;
        this.father = father;
        this.statements = statements;
        this.ogiClass = ogiClass;
        this.allStatements = allStatements;
        this.variableNum = variableNum;
        this.level = level;
        this.firstStatement = firstStatement;
        variableDestination = prefix + variableNum;
        classToOWL = Util.getOntologyOWL().getClassToOWL();
    }

    public boolean isViewAddStatement() {
        return viewAddStatement;
    }

    public boolean isDatetime() {
        return datetime;
    }

    public String getMask() {
        return mask;
    }

    public Integer getOperator() {
        return operator;
    }

    public void setOperator(Integer operator) {
        this.operator = operator;
    }

    public String getDataValue() {
        return dataValue;
    }

    public void setDataValue(String dataValue) {
        this.dataValue = dataValue;
    }

    public String getClassSelected() {
        return classSelected;
    }

    public void setClassSelected(String classSelected) {
        this.classSelected = classSelected;
    }

    public boolean isFirstStatement() {
        return firstStatement;
    }

    public boolean isCondition() {
        return condition;
    }

    public void setCondition(boolean condition) {
        this.condition = condition;
    }

    public String getDateTimeFunction() {
        return dateTimeFunction;
    }

    public void setDateTimeFunction(String dateTimeFunction) {
        this.dateTimeFunction = dateTimeFunction;
    }

    public int getVariableNum() {
        return variableNum;
    }

    public int getLevel() {
        return level;
    }

    public String getPropertySelected() {
        return propertySelected;
    }

    public void setPropertySelected(String propertySelected) {
        this.propertySelected = propertySelected;
    }

    public OGIDataProperty getDataPropertySelected() {
        return dataPropertySelected;
    }

    public OGIObjectProperty getObjectPropertySelected() {
        return objectPropertySelected;
    }

    public String getVariable() {
        return variable;
    }

    public OGIStatement getFather() {
        return father;
    }

    public List<OGIStatement> getChildren() {
        return children;
    }

    public List<OGIOwlItem> getItems() {
        return items;
    }

    public OGIClass getOgiClass() {
        return ogiClass;
    }

    public void selectDatetimeFunction() {
        if (dateTimeFunction == null || dateTimeFunction.trim().equals("")) {
            mask = "99/99/9999";
            return;
        }
        switch (dateTimeFunction) {
            case "year":
                mask = "9999";
                break;
            case "month":
                mask = "99";
                break;
            case "day":
                mask = "99";
                break;
            default:
                mask = "99/99/9999";
                break;
        }
    }

    public void selectProperty() {
        objectPropertySelected = null;
        dataPropertySelected = null;
        viewAddStatement = false;
        if (propertySelected == null) {
            return;
        }
        for (OGIObjectProperty oop : ogiClass.getObjectProperties()) {
            if (oop.getUri().equals(propertySelected)) {
                objectPropertySelected = oop;
                viewAddStatement = true;
                return;
            }
        }
        for (OGIDataProperty oop : ogiClass.getDataProperties()) {
            if (oop.getUri().equals(propertySelected)) {
                dataPropertySelected = oop;
                datetime = false;
                if (dataPropertySelected.getDatatype().toStringID().contains("dateTime")) {
                    mask = "99/99/9999";
                    datetime = true;
                }

                return;
            }
        }
    }

    public String getSPARQL() {
        if (objectPropertySelected != null) {
            if (classSelected != null) {
                return Util.generateFilterOWL(variable, classToOWL, classSelected, objectPropertySelected, variableDestination);
            } else {
                return null;
            }
        } else if (dataPropertySelected != null) {
            if (operator != null) {
                if (dataValue != null && !dataValue.trim().equals("")) {
                    String filter = generateFilter();
                    if (filter != null) {
                        return "{" + variable + " <" + dataPropertySelected.getUri() + ">" + " ?" + variableDestination + " "
                                + generateFilter();
                    } else {
                        return null;
                    }
                } else {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    protected String generateFilter() {
        if (dataPropertySelected.getDatatype().isInteger() || dataPropertySelected.getDatatype().isDouble()
                || dataPropertySelected.getDatatype().isFloat()) {
            int a;
            try {
                a = Integer.parseInt(dataValue.trim());
            } catch (Exception ex) {
                return null;
            }
            if (operator.intValue() == 0) {
                return ". FILTER (?" + variableDestination + " = " + a + ")} ";
            } else if (operator.intValue() == 1) {
                return ". FILTER (?" + variableDestination + " != " + a + ")} ";
            } else if (operator.intValue() == 2) {
                return ". FILTER (?" + variableDestination + " < " + a + ")} ";
            } else if (operator.intValue() == 3) {
                return ". FILTER (?" + variableDestination + " <= " + a + ")} ";
            } else if (operator.intValue() == 4) {
                return ". FILTER (?" + variableDestination + " > " + a + ")} ";
            } else if (operator.intValue() == 5) {
                return ". FILTER (?" + variableDestination + " >= " + a + ")} ";
            } else if (operator.intValue() == 6) {
                return ". FILTER regex (?" + variableDestination + " , '^" + a + "', 'i')} ";
            } else if (operator.intValue() == 7) {
                return ". FILTER regex (?" + variableDestination + " , '" + a + "^', 'i')} ";
            } else if (operator.intValue() == 8) {
                return ". FILTER regex (?" + variableDestination + " , '" + a + "', 'i')} ";
            } else if (operator.intValue() == 9) {
                return ". FILTER (!regex (?" + variableDestination + " , '" + a + "', 'i'))} ";
            } else {
                return null;
            }
        } else if (dataPropertySelected.getDatatype().toStringID().contains("dateTime")) {
            if (dateTimeFunction != null && !dateTimeFunction.trim().equals("")) {
                int a;
                try {
                    a = Integer.parseInt(dataValue.trim());
                } catch (Exception ex) {
                    return null;
                }
                if (operator.intValue() == 0) {
                    return ". FILTER (" + dateTimeFunction + "(?" + variableDestination + ") = " + a + ")} ";
                } else if (operator.intValue() == 1) {
                    return ". FILTER (" + dateTimeFunction + "(?" + variableDestination + ") != " + a + ")} ";
                } else if (operator.intValue() == 2) {
                    return ". FILTER (" + dateTimeFunction + "(?" + variableDestination + ") < " + a + ")} ";
                } else if (operator.intValue() == 3) {
                    return ". FILTER (" + dateTimeFunction + "(?" + variableDestination + ") <= " + a + ")} ";
                } else if (operator.intValue() == 4) {
                    return ". FILTER (" + dateTimeFunction + "(?" + variableDestination + ") > " + a + ")} ";
                } else if (operator.intValue() == 5) {
                    return ". FILTER (" + dateTimeFunction + "(?" + variableDestination + ") >= " + a + ")} ";
                } else if (operator.intValue() == 6) {
                    return ". FILTER regex (" + dateTimeFunction + "(?" + variableDestination + ") , '^" + a + "', 'i')} ";
                } else if (operator.intValue() == 7) {
                    return ". FILTER regex (" + dateTimeFunction + "(?" + variableDestination + ") , '" + a + "^', 'i')} ";
                } else if (operator.intValue() == 8) {
                    return ". FILTER regex (" + dateTimeFunction + "(?" + variableDestination + ") , '" + a + "', 'i')} ";
                } else if (operator.intValue() == 9) {
                    return ". FILTER (!regex (" + dateTimeFunction + "(?" + variableDestination + ") , '" + a + "', 'i'))} ";
                } else {
                    return null;
                }
            } else {
                Date date = Util.convertXMLDate(dataValue, "dd/MM/yyyy");
                if (date == null) {
                    return null;
                } else {
                    String dataValue = Util.getXMLDate(date);
                    if (operator.intValue() == 0) {
                        return ". FILTER (?" + variableDestination + " = '" + dataValue.trim() + "'^^xsd:dateTime)} ";
                    } else if (operator.intValue() == 1) {
                        return ". FILTER (?" + variableDestination + " != '" + dataValue.trim() + "'^^xsd:dateTime)} ";
                    } else if (operator.intValue() == 2) {
                        return ". FILTER (?" + variableDestination + " < '" + dataValue.trim() + "'^^xsd:dateTime)} ";
                    } else if (operator.intValue() == 3) {
                        return ". FILTER (?" + variableDestination + " <= '" + dataValue.trim() + "'^^xsd:dateTime)} ";
                    } else if (operator.intValue() == 4) {
                        return ". FILTER (?" + variableDestination + " > '" + dataValue.trim() + "'^^xsd:dateTime)} ";
                    } else if (operator.intValue() == 5) {
                        return ". FILTER (?" + variableDestination + " >= '" + dataValue.trim() + "'^^xsd:dateTime)} ";
                    } else if (operator.intValue() == 6) {
                        return ". FILTER regex (?" + variableDestination + " , '^" + dataValue.trim() + "'^^xsd:dateTime, 'i')} ";
                    } else if (operator.intValue() == 7) {
                        return ". FILTER regex (?" + variableDestination + " , '" + dataValue.trim() + "^'^^xsd:dateTime, 'i')} ";
                    } else if (operator.intValue() == 8) {
                        return ". FILTER regex (?" + variableDestination + " , '" + dataValue.trim() + "'^^xsd:dateTime, 'i')} ";
                    } else if (operator.intValue() == 9) {
                        return ". FILTER (!regex (?" + variableDestination + " , '" + dataValue.trim() + "'^^xsd:dateTime, 'i'))} ";
                    } else {
                        return null;
                    }
                }
            }
        } else {
            if (operator.intValue() == 0) {
                return ". FILTER (str(?" + variableDestination + ") = '" + dataValue.trim() + "')} ";
            } else if (operator.intValue() == 1) {
                return ". FILTER (str(?" + variableDestination + ") != '" + dataValue.trim() + "')} ";
            } else if (operator.intValue() == 2) {
                return ". FILTER (str(?" + variableDestination + ") < '" + dataValue.trim() + "')} ";
            } else if (operator.intValue() == 3) {
                return ". FILTER (str(?" + variableDestination + ") <= '" + dataValue.trim() + "')} ";
            } else if (operator.intValue() == 4) {
                return ". FILTER (str(?" + variableDestination + ") > '" + dataValue.trim() + "')} ";
            } else if (operator.intValue() == 5) {
                return ". FILTER (str(?" + variableDestination + ") >= '" + dataValue.trim() + "')} ";
            } else if (operator.intValue() == 6) {
                return ". FILTER regex (str(?" + variableDestination + ") , '^" + dataValue.trim() + "', 'i')} ";
            } else if (operator.intValue() == 7) {
                return ". FILTER regex (str(?" + variableDestination + ") , '" + dataValue.trim() + "^', 'i')} ";
            } else if (operator.intValue() == 8) {
                return ". FILTER regex (str(?" + variableDestination + ") , '" + dataValue.trim() + "', 'i')} ";
            } else if (operator.intValue() == 9) {
                return ". FILTER (!regex (str(?" + variableDestination + ") , '" + dataValue.trim() + "', 'i'))} ";
            } else {
                return null;
            }
        }
    }

    public void addStatetement() {
        if (objectPropertySelected == null) {
            return;
        }
        if (classSelected == null || classSelected.trim().equals("")) {
            return;
        }
        for (OGIClass oc : objectPropertySelected.getDestinations()) {
            if (oc.getUri().equals(classSelected)) {
                int position = 0;
                for (int i = 0; i < allStatements.size(); i++) {
                    OGIStatement os = allStatements.get(i);
                    if (os == this) {
                        position = i;
                        break;
                    }
                }
                position++;
                String v = "?" + variableDestination;
                variableNum++;
                if (children == null) {
                    children = new ArrayList<>();
                }
                position += calculateAddPosition();
                if (children.isEmpty()) {
                    OGIStatement st = new OGIStatement(this, statements, variableDestination, v, oc, allStatements, variableNum, level + 1, true);
                    allStatements.add(position, st);
                    children.add(st);
                } else {
                    OGIStatement st = new OGIStatement(this, statements, variableDestination, v, oc, allStatements, variableNum, level + 1, false);
                    allStatements.add(position, st);
                    children.add(st);
                }
                break;
            }
        }
    }

    protected int calculateAddPosition() {
        int result = 0;
        for (OGIStatement st : children) {
            if (st.getChildren() != null) {
                result += st.calculateAddPosition() + 1;
            } else {
                result++;
            }
        }
        return result;
    }
    
    public void deleteStatement() {        
        //delete from statement
        for (int i = 0; i < statements.size(); i++) {
            OGIStatement st = statements.get(i);
            if (st == this) {
                statements.remove(st);
                break;
            }
        }
        //delete from allstatements
        for (int i = 0; i < allStatements.size(); i++) {
            OGIStatement st = allStatements.get(i);
            if (st == this) {
                allStatements.remove(st);
                deleteAllChildren(st);
                break;
            }
        }
        //delete from father
        if (father != null) {
            if (father.getChildren() != null) {
                for (int i = 0; i < father.getChildren().size(); i++) {
                    OGIStatement st = father.getChildren().get(i);
                    if (st == this) {
                        father.getChildren().remove(st);
                        break;
                    }
                }
            }
        }
    }
    
    protected void deleteAllChildren(OGIStatement st) {
        if (st.getChildren() == null) {
            return;
        }
        for (OGIStatement s : st.getChildren()) {
            for (int i = 0; i < allStatements.size(); i++) {
                OGIStatement os = allStatements.get(i);
                if (os == s) {
                    allStatements.remove(os);
                    break;
                }
            }
            deleteAllChildren(s);
        }
    }
}
