package es.imib.cccf.web.application;

import es.imib.cccf.util.Util;
import es.imib.export.ClassToOWL;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

@ApplicationScoped
@ManagedBean
public class OntologyGuidedInputCache implements Serializable {
    protected ClassToOWL classToOWL;
    protected List<OGIClass> classes;
    protected List<OGIObjectProperty> objectProperties;
    protected List<OGIDataProperty> dataProperties;
    protected OGIClass classSelected;
    protected String uriSelected;
    protected String seriesTherapy;

    public OntologyGuidedInputCache() {
        classToOWL = Util.getOntologyOWL().getClassToOWL();
        System.out.println("Build the model");
        reload();
    }
    
    public void reload() {
        List<OWLClass> ocs = classToOWL.getOWLClasses();
        List<OWLObjectProperty> oops = classToOWL.getOWLObjectProperties();
        List<OWLDataProperty> odps = classToOWL.getOWLDataProperties();
        classes = new ArrayList<>();
        objectProperties = new ArrayList<>();
        dataProperties = new ArrayList<>();
        for (OWLClass oc : ocs) {
            OGIClass ogic = new OGIClass(oc);
            classes.add(ogic);
        }
        for (OWLObjectProperty oop : oops) {
            OGIObjectProperty ogiop = new OGIObjectProperty(oop);
            objectProperties.add(ogiop);
        }
        for (OWLDataProperty odp : odps) {
            OGIDataProperty ogidp = new OGIDataProperty(odp);
            dataProperties.add(ogidp);
        }
        for (OGIObjectProperty oop : objectProperties) {
            oop.loadSemanticInformacion(classes);
        }
        for (OGIDataProperty odp : dataProperties) {
            odp.loadSemanticInformacion(classes);
        }
        for (OGIClass oc : classes) {
            oc.loadSemanticInformacion(classes);
        }
    }

    public List<OGIClass> getClasses() {
        return classes;
    }

    public List<OGIObjectProperty> getObjectProperties() {
        return objectProperties;
    }

    public List<OGIDataProperty> getDataProperties() {
        return dataProperties;
    }

    public OGIClass getClassSelected() {
        return classSelected;
    }

    public String getUriSelected() {
        return uriSelected;
    }

    public void setUriSelected(String uriSelected) {
        this.uriSelected = uriSelected;
    }
    
    public void selectOGIClass() {
        for (OGIClass oc : classes) {
            if (oc.getUri().equals(uriSelected)) {
                classSelected = oc;
                return;
            }
        }
        classSelected = new OGIClass();
    }
}
