package es.imib.export;

import com.sun.org.apache.xerces.internal.jaxp.datatype.XMLGregorianCalendarImpl;
import es.imib.cccf.web.application.OGIClass;
import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Set;
import java.util.List;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLAnnotation;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLAxiom;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassAssertionAxiom;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLDataFactory;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLDatatype;
import org.semanticweb.owlapi.model.OWLEntity;
import org.semanticweb.owlapi.model.OWLIndividual;
import org.semanticweb.owlapi.model.OWLLiteral;
import org.semanticweb.owlapi.model.OWLNamedIndividual;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;
import org.semanticweb.owlapi.vocab.OWL2Datatype;
import org.semanticweb.owlapi.vocab.OWLRDFVocabulary;

public class ClassToOWL implements Serializable {

    protected OWLOntologyManager manager;
    protected OWLDataFactory dataFactory;
    protected OWLOntology ontology;
    protected OWLAnnotationProperty annotationId;
    protected OWLAnnotationProperty colorAnnotation;
    protected String iri;
    protected String owlPath;

    public ClassToOWL(String iri, String owlPath) throws Exception {
        this.iri = iri;
        this.owlPath = owlPath;
        loadOntology();
    }

    public OWLClass getProcedureByCode(String rootClass, String code, OWLAnnotationProperty annotationProperty) {
        IRI iriProcedure = IRI.create(iri + "#" + rootClass);
        OWLClass procedure = dataFactory.getOWLClass(iriProcedure);
        for (OWLClassExpression oc : procedure.getSubClasses(ontology)) {
            OWLClass c = oc.asOWLClass();
            for (OWLAnnotation ann : c.getAnnotations(ontology, annotationProperty)) {
                if (ann.getValue() instanceof OWLLiteral) {
                    OWLLiteral literal = (OWLLiteral) ann.getValue();
                    if (literal.getLiteral().equalsIgnoreCase(code)) {
                        return c;
                    }
                }
            }
            OWLClass result = getByCodeRecursive(code, c, annotationProperty);
            if (result != null) {
                return result;
            }
        }
        return null;
    }

    private OWLClass getByCodeRecursive(String code, OWLClass owlClass, OWLAnnotationProperty annotationProperty) {
        for (OWLClassExpression oc : owlClass.getSubClasses(ontology)) {
            OWLClass c = oc.asOWLClass();
            for (OWLAnnotation ann : c.getAnnotations(ontology, annotationProperty)) {
                if (ann.getValue() instanceof OWLLiteral) {
                    OWLLiteral literal = (OWLLiteral) ann.getValue();
                    if (literal.getLiteral().equalsIgnoreCase(code)) {
                        return c;
                    }
                }
            }
            OWLClass result = getByCodeRecursive(code, c, annotationProperty);
            if (result != null) {
                return result;
            }
        }
        return null;
    }

    private void loadOntology() throws Exception {
        manager = OWLManager.createOWLOntologyManager();
        dataFactory = manager.getOWLDataFactory();
        File file = new File(owlPath);
        ontology = manager.loadOntologyFromOntologyDocument(file);
        IRI iriCode = IRI.create(iri + "#id");
        annotationId = dataFactory.getOWLAnnotationProperty(iriCode);
        colorAnnotation = getAnnotationProperty("color");
    }

    public OWLClass getClass(String name) {
        return dataFactory.getOWLClass(IRI.create(iri + "#" + name));
    }

    public OWLClass getClassByURI(String name) {
        return dataFactory.getOWLClass(IRI.create(name));
    }

    public OWLClass getClass(String iri, String name) {
        return dataFactory.getOWLClass(IRI.create(iri + "#" + name));
    }

    public OWLDataProperty getDataProperty(String name) {
        return dataFactory.getOWLDataProperty(IRI.create(iri + "#" + name));
    }

    public OWLObjectProperty getObjetProperty(String name) {
        return dataFactory.getOWLObjectProperty(IRI.create(iri + "#" + name));
    }

    public OWLAnnotationProperty getAnnotationProperty(String name) {
        return dataFactory.getOWLAnnotationProperty(IRI.create(iri + "#" + name));
    }

    public OWLAnnotationProperty getAnnotationProperty(String iri, String name) {
        return dataFactory.getOWLAnnotationProperty(IRI.create(iri + "#" + name));
    }

    public OWLAnnotationProperty getAnnotationProperty(OWLRDFVocabulary voc) {
        return dataFactory.getOWLAnnotationProperty(voc.getIRI());
    }

    public OWLIndividual getIndividual(String name) {
        return dataFactory.getOWLNamedIndividual(IRI.create(iri + "#" + name));
    }

    public void saveOntology() throws Exception {
        manager.saveOntology(ontology);
    }

    public OWLOntologyManager getManager() {
        return manager;
    }

    public OWLDataFactory getDataFactory() {
        return dataFactory;
    }

    public OWLOntology getOntology() {
        return ontology;
    }

    public List<String> getAnnotationPropertyValueWithoutsuperclasses(OWLAnnotationProperty annotationProperty, OWLClass oc) {
        List<String> result = new ArrayList<>();
        Set<OWLAnnotation> annotations = oc.getAnnotations(ontology, annotationProperty);
        for (OWLAnnotation ann : annotations) {
            if (ann.getValue() instanceof OWLLiteral) {
                OWLLiteral l = (OWLLiteral) ann.getValue();
                result.add(l.getLiteral());
            } else {
                String s = ann.getValue().toString();
                result.add(s);
            }
        }
        return result;
    }

    public String getAnnotationPropertyOwlClass(OWLAnnotationProperty annotationProperty, OWLClass oc) {
        Set<OWLAnnotation> annotations = oc.getAnnotations(ontology, annotationProperty);
        for (OWLAnnotation ann : annotations) {
            if (ann.getValue() instanceof OWLLiteral) {
                OWLLiteral l = (OWLLiteral) ann.getValue();
                return l.getLiteral();
            }
        }
        return null;
    }

    public String getAnnotationPropertyOwlEntity(OWLAnnotationProperty annotationProperty, OWLEntity oc) {
        Set<OWLAnnotation> annotations = oc.getAnnotations(ontology, annotationProperty);
        for (OWLAnnotation ann : annotations) {
            if (ann.getValue() instanceof OWLLiteral) {
                OWLLiteral l = (OWLLiteral) ann.getValue();
                return l.getLiteral();
            } else {
                return "" + ann.getValue();
            }
        }
        return null;
    }

    public String getAnnotationPropertyOwlClass(OWLAnnotationProperty annotationProperty, OWLDataProperty oc) {
        Set<OWLAnnotation> annotations = oc.getAnnotations(ontology, annotationProperty);
        for (OWLAnnotation ann : annotations) {
            if (ann.getValue() instanceof OWLLiteral) {
                OWLLiteral l = (OWLLiteral) ann.getValue();
                return l.getLiteral();
            }
        }
        return null;
    }

    public List<String> getAnnotationPropertyValue(OWLAnnotationProperty annotationProperty, OWLClass oc) {
        List<String> result = new ArrayList<>();
        Set<OWLAnnotation> annotations = oc.getAnnotations(ontology, annotationProperty);
        while (!annotations.isEmpty()) {
            for (OWLAnnotation ann : annotations) {
                String s = ann.getValue().toString();
                result.add(0, s);
                break;
            }
            boolean finded = false;
            for (OWLClassExpression oce : oc.getSuperClasses(ontology)) {
                if (oce instanceof OWLClass) {
                    OWLClass occ = oce.asOWLClass();
                    oc = occ;
                    annotations = oc.getAnnotations(ontology, annotationProperty);
                    finded = true;
                    break;
                }
            }
            if (!finded) {
                break;
            }
        }
        return result;
    }

    public void createIndividual(String name, String id, OWLClass oc) {
        IRI ind = IRI.create(iri + "#" + name);
        OWLNamedIndividual individual = dataFactory.getOWLNamedIndividual(ind);
        OWLAnnotation idann = dataFactory.getOWLAnnotation(annotationId, dataFactory.getOWLLiteral(id));
        OWLAxiom axId = dataFactory.getOWLAnnotationAssertionAxiom(individual.getIRI(), idann);
        manager.addAxiom(ontology, axId);
        OWLClassAssertionAxiom axiom = dataFactory.getOWLClassAssertionAxiom(oc, individual);
        manager.addAxiom(ontology, axiom);
    }

    public List<OWLIndividual> getIndividualsForObjectProperty(OWLIndividual individual, OWLObjectProperty objectProperty) {
        List<OWLIndividual> inds = new ArrayList<>();
        for (OWLIndividual ind : individual.getObjectPropertyValues(objectProperty, ontology)) {
            inds.add(ind);
        }
        return inds;
    }
    
    public List<OWLClass> getIndividualTypes(OWLIndividual individual) {
        List<OWLClass> ocs = new ArrayList<>();
        for (OWLClassExpression oce : individual.getTypes(ontology)) {
            if (oce instanceof OWLClass) {
                ocs.add(oce.asOWLClass());
            }
        }
        return ocs;
    }

    public void createStringDataProperty(OWLDataProperty dataProperty, OWLIndividual individual, String value) {
        OWLAxiom axiom = dataFactory.getOWLDataPropertyAssertionAxiom(dataProperty, individual, value);
        manager.addAxiom(ontology, axiom);
    }

    public void createLongDataProperty(OWLDataProperty dataProperty, OWLIndividual individual, Long value) {
        OWLDatatype datatype = dataFactory.getIntegerOWLDatatype();
        OWLLiteral literal = dataFactory.getOWLLiteral(value.toString(), datatype);
        OWLAxiom axiom = dataFactory.getOWLDataPropertyAssertionAxiom(dataProperty, individual, literal);
        manager.addAxiom(ontology, axiom);
    }

    public void createDoubleDataProperty(OWLDataProperty dataProperty, OWLIndividual individual, Double value) {
        OWLDatatype datatype = dataFactory.getDoubleOWLDatatype();
        OWLLiteral literal = dataFactory.getOWLLiteral(value.toString(), datatype);
        OWLAxiom axiom = dataFactory.getOWLDataPropertyAssertionAxiom(dataProperty, individual, literal);
        manager.addAxiom(ontology, axiom);
    }

    public void createDatetimeDateProperty(OWLDataProperty dataProperty, OWLIndividual individual, Date value) {
        OWL2Datatype datatype = OWL2Datatype.XSD_DATE_TIME;
        GregorianCalendar gc = new GregorianCalendar();
        gc.setTime(value);
        XMLGregorianCalendarImpl xml = new XMLGregorianCalendarImpl(gc);
        OWLLiteral literal = dataFactory.getOWLLiteral(xml.toXMLFormat(), datatype);
        OWLAxiom axiom = dataFactory.getOWLDataPropertyAssertionAxiom(dataProperty, individual, literal);
        manager.addAxiom(ontology, axiom);
    }

    public void createObjectProperty(OWLIndividual source, OWLIndividual destination, OWLObjectProperty property) {
        OWLAxiom axiom = dataFactory.getOWLObjectPropertyAssertionAxiom(property, source, destination);
        manager.addAxiom(ontology, axiom);
    }

    public void saveOntologyAs(String output) throws Exception {
        IRI irioutput = IRI.create(output);
        manager.saveOntology(ontology, irioutput);
    }

    public List<OWLClass> getAllSheets(OWLClass father) {
        List<OWLClass> results = new ArrayList<>();
        for (OWLClassExpression oce : father.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                if (c.getSubClasses(ontology).isEmpty()) {
                    results.add(c);
                } else {
                    loadRecursiveAllSheets(father, results);
                }
            }
        }
        return results;
    }

    protected void loadRecursiveAllSheets(OWLClass father, List<OWLClass> results) {
        for (OWLClassExpression oce : father.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                if (c.getSubClasses(ontology).isEmpty()) {
                    results.add(c);
                } else {
                    loadRecursiveAllSheets(c, results);
                }
            }
        }
    }

    public List<OWLClass> getAllSubclasses(String f) {
        OWLClass father = getClass(f);
        List<OWLClass> results = new ArrayList<>();
        for (OWLClassExpression oce : father.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                results.add(c);
                loadRecursiveAllSubclasses(c, results);
            }
        }
        return results;
    }

    public List<OWLClass> getAllSubclasses(OWLClass father) {
        List<OWLClass> results = new ArrayList<>();
        for (OWLClassExpression oce : father.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                results.add(c);
                loadRecursiveAllSubclasses(c, results);
            }
        }
        return results;
    }

    public List<OWLClass> getAllSuperclasses(OWLClass father) {
        List<OWLClass> results = new ArrayList<>();
        for (OWLClassExpression oce : father.getSuperClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                results.add(c);
                loadRecursiveAllSuperclasses(c, results);
            }
        }
        return results;
    }

    public List<OWLClass> getAllSubclassesNoRecursive(OWLClass father) {
        List<OWLClass> results = new ArrayList<>();
        for (OWLClassExpression oce : father.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                results.add(c);
            }
        }
        return results;
    }

    protected void loadRecursiveAllSubclasses(OWLClass father, List<OWLClass> results) {
        for (OWLClassExpression oce : father.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                results.add(c);
                loadRecursiveAllSubclasses(c, results);
            }
        }
    }

    protected void loadRecursiveAllSuperclasses(OWLClass father, List<OWLClass> results) {
        for (OWLClassExpression oce : father.getSuperClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                results.add(c);
                loadRecursiveAllSuperclasses(c, results);
            }
        }
    }

    public OWLIndividual findIndividualByClassAndProperty(OWLClass oc, OWLDataProperty property, String propertyName) {
        for (OWLIndividual ind : oc.getIndividuals(ontology)) {
            for (OWLLiteral li : ind.getDataPropertyValues(property, ontology)) {
                if (li.getLiteral().equalsIgnoreCase(propertyName)) {
                    return ind;
                }
            }
        }
        return findIndividualRevursiveByClassAndProperty(oc, property, propertyName);
    }

    protected OWLIndividual findIndividualRevursiveByClassAndProperty(OWLClass oc, OWLDataProperty property, String propertyName) {
        for (OWLClassExpression oce : oc.getSubClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass c = oce.asOWLClass();
                for (OWLIndividual ind : c.getIndividuals(ontology)) {
                    for (OWLLiteral li : ind.getDataPropertyValues(property, ontology)) {
                        if (li.getLiteral().equalsIgnoreCase(propertyName)) {
                            return ind;
                        }
                    }
                }
                OWLIndividual oi = findIndividualRevursiveByClassAndProperty(c, property, propertyName);
                if (oi != null) {
                    return oi;
                }
            }
        }
        return null;
    }

    public OWLClass getOWLClassSuperclassOverListClass(OWLClass oc, List<OWLClass> list) {
        for (OWLClassExpression oce : oc.getSuperClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass o = oce.asOWLClass();
                for (OWLClass ocl : list) {
                    if (ocl.toStringID().equals(o.toStringID())) {
                        return ocl;
                    }
                }
                OWLClass result = getOWLClassSuperclassOverListClassRecursive(o, list);
                if (result != null) {
                    return result;
                }
            }
        }
        return null;
    }

    public OWLClass getOWLClassSuperclassOverListClassRecursive(OWLClass oc, List<OWLClass> list) {
        for (OWLClassExpression oce : oc.getSuperClasses(ontology)) {
            if (oce instanceof OWLClass) {
                OWLClass o = oce.asOWLClass();
                for (OWLClass ocl : list) {
                    if (ocl.toStringID().equals(o.toStringID())) {
                        return ocl;
                    }
                }
                OWLClass result = getOWLClassSuperclassOverListClassRecursive(o, list);
                if (result != null) {
                    return result;
                }
            }
        }
        return null;
    }

    public List<OWLClass> getOWLClasses() {
        List<OWLClass> ocs = new ArrayList<>();
        for (OWLClass oc : ontology.getClassesInSignature()) {
            ocs.add(oc);
        }
        return ocs;
    }

    public List<OWLObjectProperty> getOWLObjectProperties() {
        List<OWLObjectProperty> oops = new ArrayList<>();
        for (OWLObjectProperty oop : ontology.getObjectPropertiesInSignature()) {
            oops.add(oop);
        }
        return oops;
    }

    public List<OWLDataProperty> getOWLDataProperties() {
        List<OWLDataProperty> odps = new ArrayList<>();
        for (OWLDataProperty odp : ontology.getDataPropertiesInSignature()) {
            odps.add(odp);
        }
        return odps;
    }
    
    public String getColors(String fragment) {
        OWLClass father = getClass(fragment);        
        List<OWLClass> ocs = getAllSubclasses(father);
        if (ocs == null || ocs.isEmpty()) {
            return null;
        } else {
            String result = null;
            for (OWLClass oc : ocs) {
                OGIClass ogic = new OGIClass(oc);
                if (ogic.getColor() != null) {
                    if (result == null) {
                        result = ogic.getColor();
                    } else {
                        result += ", " + ogic.getColor();
                    }
                }
            }
            return result;
        }
    }
    
    public String getColor(String fragment) {
        OWLClass oc = getClass(fragment);
        return getAnnotationPropertyOwlEntity(colorAnnotation, oc);
    }
}
