package es.imib.export;

import es.imib.persistence.Course;
import java.util.Date;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLIndividual;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class CourseBean extends OWLMigrationBean<Course> implements Comparable<CourseBean> {

    protected PatientBean patientBean;

    public CourseBean(Course bean, PatientBean patientBean) {
        super(bean);
        this.patientBean = patientBean;
    }

    @Override
    public void loadSemanticInformation() {
        if (bean.getCreationDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("creationDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getCreationDate().getTime()));
        }
        if (bean.getUpdateDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("editionDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getUpdateDate().getTime()));
        }
        if (bean.getExaminationDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("examinationDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getExaminationDate().getTime()));
        }
        //ObjectProperties
        if (bean.getCreateDepartment() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasCreationClinicalDepartment");
            OWLDataProperty dataProperty = classToOWL.getDataProperty("designation");
            OWLIndividual department = classToOWL.findIndividualByClassAndProperty(classToOWL.getClass("ClinicalDepartment"), dataProperty, bean.getCreateDepartment());
            classToOWL.createObjectProperty(individual, department, property);
        }
        if (bean.getUpdateDepartment() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasEditionClinicalDepartment");
            OWLDataProperty dataProperty = classToOWL.getDataProperty("designation");
            OWLIndividual department = classToOWL.findIndividualByClassAndProperty(classToOWL.getClass("ClinicalDepartment"), dataProperty, bean.getCreateDepartment());
            classToOWL.createObjectProperty(individual, department, property);
        }
        if (bean.getPatientNdx() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasPatient");
            classToOWL.createObjectProperty(individual, patientBean.getIndividual(), property);
        }
    }

    public void loadSecondarySemanticInformation() {
        //ObjectProperty                        
        //hasTherapy -- Careful
        for (DiagnosisBean diagnosisBean : patientBean.getDiagnosises()) {
            if (diagnosisBean.getBean().getDiagnosisNdx().longValue() == bean.getDiagnosisNdx().longValue()) {
                OWLObjectProperty property = classToOWL.getObjetProperty("hasDiagnosis");
                classToOWL.createObjectProperty(individual, diagnosisBean.getIndividual(), property);
            }
        }
        //hasDiseaseCourse -- Careful
        if (bean.getPatientConditionNdx() != null) {
            for (PatientConditionBean pcb : patientBean.getConditions()) {
                if (pcb.getBean().getConditionNdx().longValue() == bean.getPatientConditionNdx().longValue()) {
                    OWLObjectProperty property = classToOWL.getObjetProperty("hasCondition");
                    classToOWL.createObjectProperty(individual, pcb.getIndividual(), property);
                }
            }
        }
    }

    @Override
    public int compareTo(CourseBean o) {
        return bean.getExaminationDate().compareTo(o.getBean().getExaminationDate());
    }
    
    
}
