package es.imib.export;

import es.imib.persistence.Diagnosis;
import es.imib.persistence.daos.CourseDAO;
import es.imib.persistence.daos.TherapyDAO;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLIndividual;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class DiagnosisBean extends OWLMigrationBean<Diagnosis> implements Comparable<DiagnosisBean> {

    protected List<TherapyBean> therapies;
    protected List<CourseBean> courses;
    protected TherapyDAO therapyDAO;
    protected CourseDAO courseDAO;
    protected PatientBean patientBean;

    public DiagnosisBean(Diagnosis bean, PatientBean patientBean) {
        super(bean);
        this.patientBean = patientBean;
        therapies = new ArrayList<>();
        courses = new ArrayList<>();
        therapyDAO = TherapyDAO.getTherapyDAO();
        courseDAO = CourseDAO.getCourseDAO();
    }

    @Override
    public void loadSemanticInformation() {
        //DataProperty
        if (bean.getCreationDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("creationDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getCreationDate().getTime()));
        }
        if (bean.getUpdateDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("editionDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getUpdateDate().getTime()));
        }
        if (bean.getDiagnosisDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("referenceDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getDiagnosisDate().getTime()));
        }
        //creationDate
        //editionDate
        //referenceDate

        //ObjectProperty
        if (bean.getCreateDepartment() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasCreationClinicalDepartment");
            OWLDataProperty dataProperty = classToOWL.getDataProperty("designation");
            OWLIndividual department = classToOWL.findIndividualByClassAndProperty(classToOWL.getClass("ClinicalDepartment"), dataProperty, bean.getCreateDepartment());
            classToOWL.createObjectProperty(individual, department, property);
        }
        if (bean.getUpdateDepartment() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasEditionClinicalDepartment");
            OWLDataProperty dataProperty = classToOWL.getDataProperty("designation");
            OWLIndividual department = classToOWL.findIndividualByClassAndProperty(classToOWL.getClass("ClinicalDepartment"), dataProperty, bean.getCreateDepartment());
            classToOWL.createObjectProperty(individual, department, property);
        }
        if (bean.getPatientNdx() != null) {
            if (patientBean != null) {
                OWLObjectProperty property = classToOWL.getObjetProperty("hasPatient");
                classToOWL.createObjectProperty(individual, patientBean.getIndividual(), property);
            }
        }
        if (bean.getGrading() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasGrading");
            OWLClass oc = classToOWL.getProcedureByCode("Grading", bean.getGrading(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getcTnmStadium() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasClinicalStage");
            OWLClass oc = classToOWL.getProcedureByCode("Stage", bean.getcTnmStadium(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getcTnmMWert() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasClinicalDistantMetastasis");
            OWLClass oc = classToOWL.getProcedureByCode("DistantMetastasis", bean.getcTnmMWert(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getcTnmNWert() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasClinicalRegionalLymphNodes");
            OWLClass oc = classToOWL.getProcedureByCode("RegionalLymphNodes", bean.getcTnmNWert(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getcTnmTWert() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasClinicalPrimaryTumor");
            OWLClass oc = classToOWL.getProcedureByCode("PrimaryTumor", bean.getcTnmTWert(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getpTnmStadium() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasPathologicalStage");
            OWLClass oc = classToOWL.getProcedureByCode("Stage", bean.getpTnmStadium(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getpTnmMWert() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasPathologicalDistantMetastasis");
            OWLClass oc = classToOWL.getProcedureByCode("DistantMetastasis", bean.getpTnmMWert(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getpTnmNWert() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasPathologicalRegionalLymphNodes");
            OWLClass oc = classToOWL.getProcedureByCode("RegionalLymphNodes", bean.getpTnmNWert(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        if (bean.getpTnmTWert() != null) {
            OWLObjectProperty property = classToOWL.getObjetProperty("hasPathologicalPrimaryTumor");
            OWLClass oc = classToOWL.getProcedureByCode("PrimaryTumor", bean.getpTnmTWert(), classToOWL.getAnnotationProperty("code"));
            classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
            OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
            classToOWL.createObjectProperty(individual, gr, property);
        }
        //hasCreationClinicalDepartment
        //hasEditionClinicalDepartment
        if (bean.getIcd10() != null) {
            String id = bean.getIcd10().replaceAll("\\.", "_");
            if (id.endsWith("00")) {
                id = id.substring(0, id.length() - 1);
            }
            OWLClass oc = classToOWL.getClass("http://www.owl-ontologies.com/unnamed.owl", id);
            if (oc == null) {
                System.out.println("It can't find the ICD10 code: " + bean.getIcd10());
            } else {
                OWLIndividual oi = classToOWL.getIndividual("IND_" + id);
                classToOWL.createIndividual("IND_" + id, id, oc);
                classToOWL.createObjectProperty(individual, oi, classToOWL.getObjetProperty("hasICD10_Classification"));
                //Organ
                List<OWLIndividual> inds = classToOWL.getIndividualsForObjectProperty(oi, classToOWL.getObjetProperty("hasAnatomicalStructure"));
                if (!inds.isEmpty()) {
                    OWLIndividual anatomicalStructureInd = inds.get(0);
                    List<OWLClass> ocs = classToOWL.getIndividualTypes(anatomicalStructureInd);
                    if (!ocs.isEmpty()) {
                        OWLClass organClass = ocs.get(0);
                        String nOrgan = "IND_" + organClass.getIRI().getFragment();
                        classToOWL.createIndividual(nOrgan, organClass.getIRI().getFragment(), organClass);
                        OWLIndividual iorgan = classToOWL.getIndividual(nOrgan);
                        classToOWL.createObjectProperty(individual, iorgan, classToOWL.getObjetProperty("hasAnatomicalStructure"));
                    }
                }
                /*List<OWLClass> ocs = classToOWL.getAllSubclassesNoRecursive(classToOWL.getClass("Organ"));
                 OWLClass organClass = classToOWL.getOWLClassSuperclassOverListClass(oc, ocs);
                 if (organClass != null) {
                 String nOrgan = "IND_ " + organClass.getIRI().getFragment();
                 classToOWL.createIndividual(nOrgan, nOrgan, organClass);
                 OWLIndividual iorgan = classToOWL.getIndividual(nOrgan);
                 classToOWL.createObjectProperty(individual, iorgan, classToOWL.getObjetProperty("hasAnatomicalStructure"));
                 }*/
                //TumorType
                inds = classToOWL.getIndividualsForObjectProperty(oi, classToOWL.getObjetProperty("hasTumorType"));
                if (!inds.isEmpty()) {
                    OWLIndividual anatomicalStructureInd = inds.get(0);
                    List<OWLClass> ocs = classToOWL.getIndividualTypes(anatomicalStructureInd);
                    if (!ocs.isEmpty()) {
                        OWLClass tumorTypeClass = ocs.get(0);
                        String nOrgan = "IND_" + tumorTypeClass.getIRI().getFragment();
                        classToOWL.createIndividual(nOrgan, tumorTypeClass.getIRI().getFragment(), tumorTypeClass);
                        OWLIndividual iorgan = classToOWL.getIndividual(nOrgan);
                        classToOWL.createObjectProperty(individual, iorgan, classToOWL.getObjetProperty("hasTumorType"));
                    }
                }
                /*ocs = classToOWL.getAllSubclassesNoRecursive(classToOWL.getClass("TumorType"));
                 OWLClass tumorTypeClass = classToOWL.getOWLClassSuperclassOverListClass(oc, ocs);
                 if (tumorTypeClass != null) {
                 String nOrgan = "IND_ " + tumorTypeClass.getIRI().getFragment();
                 classToOWL.createIndividual(nOrgan, nOrgan, tumorTypeClass);
                 OWLIndividual iorgan = classToOWL.getIndividual(nOrgan);
                 classToOWL.createObjectProperty(individual, iorgan, classToOWL.getObjetProperty("hasTumorType"));
                 }*/
                //CancerType
                inds = classToOWL.getIndividualsForObjectProperty(oi, classToOWL.getObjetProperty("hasPathologicalStructure"));
                if (!inds.isEmpty()) {
                    OWLIndividual anatomicalStructureInd = inds.get(0);
                    List<OWLClass> ocs = classToOWL.getIndividualTypes(anatomicalStructureInd);
                    if (!ocs.isEmpty()) {
                        OWLClass cancerTypeClass = ocs.get(0);
                        String nOrgan = "IND_" + cancerTypeClass.getIRI().getFragment();
                        classToOWL.createIndividual(nOrgan, cancerTypeClass.getIRI().getFragment(), cancerTypeClass);
                        OWLIndividual iorgan = classToOWL.getIndividual(nOrgan);
                        classToOWL.createObjectProperty(individual, iorgan, classToOWL.getObjetProperty("hasPathologicalStructure"));
                    }
                }
                /*ocs = classToOWL.getAllSubclassesNoRecursive(classToOWL.getClass("CancerType"));
                OWLClass cancerTypeClass = classToOWL.getOWLClassSuperclassOverListClass(oc, ocs);
                if (cancerTypeClass != null) {
                    String nOrgan = "IND_ " + cancerTypeClass.getIRI().getFragment();
                    classToOWL.createIndividual(nOrgan, nOrgan, cancerTypeClass);
                    OWLIndividual iorgan = classToOWL.getIndividual(nOrgan);
                    classToOWL.createObjectProperty(individual, iorgan, classToOWL.getObjetProperty("hasPathologicalStructure"));
                }*/
            }
        }
        //hasICD10_Classification
    }

    public void loadSecondarySemanticInformation() {
        //ObjectProperty                        
        //hasTherapy -- Careful
        List<TherapyBean> misTherapies = new ArrayList<>();
        List<CourseBean> misCourses = new ArrayList<>();
        for (TherapyBean therapyBean : patientBean.getTherapies()) {
            if (therapyBean.getBean().getDianosisNdx().longValue() == bean.getDiagnosisNdx().longValue()) {
                if (therapyBean.getIndividual() != null) {
                    OWLObjectProperty property = classToOWL.getObjetProperty("hasTherapy");
                    classToOWL.createObjectProperty(individual, therapyBean.getIndividual(), property);
                    misTherapies.add(therapyBean);
                }
            }
        }
        Collections.sort(misTherapies);
        int order = 0;
        Timestamp ts = null;
        for (TherapyBean tb : misTherapies) {
            if (ts == null) {
                ts = tb.getBean().getStartDate();
                tb.addOrder(order);
                continue;
            }
            if (ts.compareTo(tb.getBean().getStartDate()) == 0) {
                tb.addOrder(order);
            } else {
                ts = tb.getBean().getStartDate();
                order++;
                tb.addOrder(order);
            }
        }
        if (!misTherapies.isEmpty()) {
            TherapyBean tb = misTherapies.get(misTherapies.size() - 1);
            tb.addLast();
        }
        //hasDiseaseCourse -- Careful
        for (CourseBean courseBean : patientBean.getCourses()) {
            if (courseBean.getBean().getDiagnosisNdx().longValue() == bean.getDiagnosisNdx().longValue()) {
                OWLObjectProperty property = classToOWL.getObjetProperty("hasDiseaseCourse");
                classToOWL.createObjectProperty(individual, courseBean.getIndividual(), property);
                misCourses.add(courseBean);
            }
        }
        Collections.sort(misCourses);
        order = 0;
        ts = null;
        for (CourseBean tb : misCourses) {
            if (ts == null) {
                ts = tb.getBean().getExaminationDate();
                tb.addOrder(order);
                continue;
            }
            if (ts.compareTo(tb.getBean().getExaminationDate()) == 0) {
                tb.addOrder(order);
            } else {
                ts = tb.getBean().getExaminationDate();
                order++;
                tb.addOrder(order);
            }
        }
        if (!misCourses.isEmpty()) {
            CourseBean cb = misCourses.get(misCourses.size() - 1);
            cb.addLast();
        }
    }

    @Override
    public int compareTo(DiagnosisBean o) {
        return bean.getDiagnosisDate().compareTo(o.getBean().getDiagnosisDate());
    }
}
