package es.imib.export;

import es.imib.cccf.test.TestLoadPatients;
import es.imib.persistence.Course;
import es.imib.persistence.Diagnosis;
import es.imib.persistence.Patient;
import es.imib.persistence.PatientCondition;
import es.imib.persistence.Therapy;
import es.imib.persistence.daos.CourseDAO;
import es.imib.persistence.daos.DiagnosisDAO;
import es.imib.persistence.daos.PatientConditionDAO;
import es.imib.persistence.daos.TherapyDAO;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLObjectProperty;

public class PatientBean extends OWLMigrationBean<Patient> {

    protected List<DiagnosisBean> diagnosises;
    protected List<CourseBean> courses;
    protected List<TherapyBean> therapies;
    protected List<PatientConditionBean> conditions;
    protected DiagnosisDAO diagnosisDAO;
    protected CourseDAO courseDAO;
    protected TherapyDAO therapyDAO;
    protected PatientConditionDAO patientConditionDAO;

    public PatientBean(Patient patient) {
        super(patient);
        diagnosisDAO = DiagnosisDAO.getDiagnosisDAO();
        courseDAO = CourseDAO.getCourseDAO();
        therapyDAO = TherapyDAO.getTherapyDAO();
        patientConditionDAO = PatientConditionDAO.getPatientConditionDAO();
        diagnosises = new ArrayList<>();
        courses = new ArrayList<>();
        conditions = new ArrayList<>();
        List<Diagnosis> ds = diagnosisDAO.findByPatient(bean.getPatientNdx(), null, false);
        for (Diagnosis d : ds) {
            DiagnosisBean db = new DiagnosisBean(d, this);
            diagnosises.add(db);
        }
        List<Course> cs = courseDAO.findByPatient(bean.getPatientNdx(), null, false);
        for (Course c : cs) {
            CourseBean cb = new CourseBean(c, this);
            courses.add(cb);
        }
        List<PatientCondition> pcs = patientConditionDAO.findByPatient(bean.getPatientNdx(), null, false);
        for (PatientCondition pc : pcs) {
            PatientConditionBean pcb = new PatientConditionBean(pc);
            conditions.add(pcb);
        }
        List<Therapy> ts = therapyDAO.findByPatient(bean.getPatientNdx(), null, false);
        therapies = new ArrayList<>();
        for (Therapy t : ts) {
            TherapyBean tb = new TherapyBean(t, this);
            therapies.add(tb);
        }
    }

    public List<DiagnosisBean> getDiagnosises() {
        return diagnosises;
    }

    public List<CourseBean> getCourses() {
        return courses;
    }

    public List<PatientConditionBean> getConditions() {
        return conditions;
    }

    public List<TherapyBean> getTherapies() {
        return therapies;
    }

    @Override
    public void loadSemanticInformation() {
        loadIndividual();
        //Data Property
        // gender        
        if (bean.getSalutation() != null) {
            OWLDataProperty property = classToOWL.getDataProperty("gender");
            if (bean.getSalutation().startsWith("F")) {
                classToOWL.createStringDataProperty(property, individual, "F");
            } else if (bean.getSalutation().startsWith("H")) {
                classToOWL.createStringDataProperty(property, individual, "M");
            }
        }
        OWLDataProperty property = classToOWL.getDataProperty("age");
        classToOWL.createLongDataProperty(property, individual, bean.getYearsOld());
        //ObjectProperty
        OWLClass patientConditionClass = classToOWL.getClass("PatientCondition");
        OWLObjectProperty hasCondition = classToOWL.getObjetProperty("hasCondition");
        for (PatientConditionBean pc : conditions) {
            pc.setAge(bean.getYearsOld());
            String n = "PatientCondition_" + pc.getBean().getConditionNdx();
            classToOWL.createIndividual(n, pc.getBean().getConditionNdx().toString(), patientConditionClass);
            pc.setName(n);
            pc.setClassToOWL(classToOWL);
            pc.loadIndividual();
            pc.loadSemanticInformation();
            classToOWL.createObjectProperty(individual, pc.getIndividual(), hasCondition);
        }
        // hasCondition
        OWLClass diagnosisClass = classToOWL.getClass("Diagnosis");
        OWLObjectProperty hasDiagnosis = classToOWL.getObjetProperty("hasDiagnosis");
        for (DiagnosisBean db : diagnosises) {
            String n = "Diagnosis_" + db.getBean().getDiagnosisNdx();
            classToOWL.createIndividual(n, db.getBean().getDiagnosisNdx().toString(), diagnosisClass);
            db.setName(n);
            db.setClassToOWL(classToOWL);
            db.loadIndividual();
            db.loadSemanticInformation();
            classToOWL.createObjectProperty(individual, db.getIndividual(), hasDiagnosis);
        }
        // hasDiagnosis        
        OWLObjectProperty hasDiseaseCourse = classToOWL.getObjetProperty("hasDiseaseCourse");
        for (CourseBean db : courses) {
            OWLAnnotationProperty acronym = classToOWL.getAnnotationProperty("acronym");
            OWLClass courseClass = classToOWL.getProcedureByCode("DiseaseCourse", db.getBean().getTotalTumorStatus(), acronym);
            String n = "DiseaseCourse_" + db.getBean().getCourseNdx();
            classToOWL.createIndividual(n, db.getBean().getCourseNdx().toString(), courseClass);
            db.setName(n);
            db.setClassToOWL(classToOWL);
            db.loadIndividual();
            db.loadSemanticInformation();
            classToOWL.createObjectProperty(individual, db.getIndividual(), hasDiseaseCourse);
        }
        // hasDiseaseCourse

        OWLObjectProperty hasTherapy = classToOWL.getObjetProperty("hasTherapy");
        for (TherapyBean db : therapies) {
            OWLClass therapyClass;
            if (db.getBean().getDetailText() != null) {
                therapyClass = classToOWL.getClass(TestLoadPatients.mapTherapies.get(db.getBean().getDetailText()));
            } else {
                continue;
            }
            String n = "Therapy_" + db.getBean().getMeasureNdx();
            classToOWL.createIndividual(n, db.getBean().getMeasureNdx().toString(), therapyClass);
            db.setName(n);
            db.setClassToOWL(classToOWL);
            db.loadIndividual();
            db.loadSemanticInformation();
            classToOWL.createObjectProperty(individual, db.getIndividual(), hasTherapy);
        }
        Collections.sort(diagnosises);
        int order = 0;
        Timestamp ts = null;
        for (DiagnosisBean db : diagnosises) {
            db.loadSecondarySemanticInformation();
            if (ts == null) {
                ts = db.getBean().getDiagnosisDate();
                db.addOrder(order);
                continue;
            }
            if (ts.compareTo(db.getBean().getDiagnosisDate()) == 0) {
                db.addOrder(order);
            } else {
                ts = db.getBean().getDiagnosisDate();
                order++;
                db.addOrder(order);
            }
        }
        if (!diagnosises.isEmpty()) {
            DiagnosisBean db = diagnosises.get(diagnosises.size() - 1);
            db.addLast();
        }
        for (CourseBean cb : courses) {
            cb.loadSecondarySemanticInformation();
        }
        for (TherapyBean tb : therapies) {
            if (tb.getIndividual() != null) {
                tb.loadSecondarySemanticInformation();
            }
        }
        // hasTherapy
    }
}
