package es.imib.export;

import es.imib.persistence.PatientCondition;
import java.util.Date;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataProperty;
import org.semanticweb.owlapi.model.OWLIndividual;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.vocab.OWLRDFVocabulary;

public class PatientConditionBean extends OWLMigrationBean<PatientCondition> {

    protected Long age;

    public PatientConditionBean(PatientCondition bean) {
        super(bean);
    }

    public Long getAge() {
        return age;
    }

    public void setAge(Long age) {
        this.age = age;
    }

    @Override
    public void loadSemanticInformation() {
        //Dataproperty
        //age
        if (age != null) {
            OWLDataProperty ageProperty = classToOWL.getDataProperty("age");
            classToOWL.createLongDataProperty(ageProperty, individual, age);
        }
        //referenceDate
        if (bean.getReferenceDate() != null) {
            OWLDataProperty referenceDate = classToOWL.getDataProperty("referenceDate");
            classToOWL.createDatetimeDateProperty(referenceDate, individual, new Date(bean.getReferenceDate().getTime()));
        }

        //ObjectProperty
        //hasASA_Classification
        if (bean.getAsaIndex() != null) {
            if (bean.getAsaIndex().intValue() != 0) {
                OWLObjectProperty asaProperty = classToOWL.getObjetProperty("hasASA_Classification");
                OWLIndividual ind = classToOWL.getIndividual("ASA_ClassificationSystem_" + bean.getAsaIndex());
                classToOWL.createObjectProperty(individual, ind, asaProperty);
            }
        }
        //hasMenopauseStatus
        if (bean.getMenopauseStatus() != null) {
            OWLObjectProperty menopauseProperty = classToOWL.getObjetProperty("hasMenopauseStatus");
            OWLIndividual ind = classToOWL.getIndividual("MenopauseStatus_" + bean.getMenopauseStatus());
            classToOWL.createObjectProperty(individual, ind, menopauseProperty);
        }
        if (bean.getKarnofskyIndex() != null) {
            OWLObjectProperty ks = classToOWL.getObjetProperty("hasKarnofskyScoring");
            OWLClass oc = classToOWL.getProcedureByCode("KarnofskyScoring", bean.getKarnofskyIndex().toString(), classToOWL.getAnnotationProperty("code"));
            if (oc != null) {
                classToOWL.createIndividual("IND_" + oc.getIRI().getFragment(), oc.getIRI().getFragment(), oc);
                OWLIndividual gr = classToOWL.getIndividual("IND_" + oc.getIRI().getFragment());
                classToOWL.createObjectProperty(individual, gr, ks);
                OWLAnnotationProperty oap = classToOWL.getAnnotationProperty(OWLRDFVocabulary.RDFS_LABEL);
                String label = classToOWL.getAnnotationPropertyOwlEntity(oap, oc);
                classToOWL.createStringDataProperty(classToOWL.getDataProperty("designation"), individual, label);
            }
        }
    }

}
