package es.imib.jsf;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class JSFUtil implements Serializable {
    static char hexDigits[] = {'0', '1', '2', '3', '4', '5', '6', '7',
        '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    public static String getActualPage() {
        String p = FacesContext.getCurrentInstance().getExternalContext().getRequestServletPath();
        HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
        String s = request.getQueryString();
        if (s != null) {
            s = "?" + s;
        } else {
            s = "";
        }
        return p + s;
    }
    
    public static String getString(String s) {
        if (s == null || s.trim().equals("")) {
            return null;
        }
        s = s.trim();
        return s;
    }
    
    public static String getMultilanguageErrorMessage(String locale, String key) {
        Locale lc = new Locale(locale);
        ResourceBundle rb = ResourceBundle.getBundle("es.fis.configuration.errors", lc);        
        return rb.getString(key);
    }
    
    public static String getMultilanguageMessage(String locale, String key) {
        Locale lc = new Locale(locale);
        ResourceBundle rb = ResourceBundle.getBundle("es.fis.configuration.messages", lc);        
        return rb.getString(key);
    }

    public static String getServletPath() {
        return FacesContext.getCurrentInstance().getExternalContext().getRequestServletPath();
    }
    
    public static String getDomain() {
        return FacesContext.getCurrentInstance().getExternalContext().getRequestServerName();
    }

    public static String getStringHttpRequestParameter(String name) {
        return FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get(name);
    }

    public static List<String> getStringRequestParameter(String name) {
        ArrayList<String> result = new ArrayList();
        String[] parameters = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterValuesMap().get(name);
        if (parameters != null) {
            for (String s : parameters) {
                result.add(s);
            }
        }
        return result;

    }

    public static Integer getIntegerHttpRequestParameter(String name) {
        Integer result;
        String s = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get(name);
        if (s == null) {
            return null;
        }
        try {
            result = Integer.parseInt(s);
        } catch (Exception ex) {
            return null;
        }
        return result;
    }
    
    public static Long getLongHttpRequestParameter(String name) {
        Long result;
        String s = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get(name);
        if (s == null) {
            return null;
        }
        try {
            result = Long.parseLong(s);
        } catch (Exception ex) {
            return null;
        }
        return result;
    }

    public static Integer getIntegerHttpRequestParameter(String name, HttpServletRequest request) {
        Integer result;
        String s = request.getParameter(name);
        if (s == null) {
            return null;
        }
        try {
            result = Integer.parseInt(s);
        } catch (Exception ex) {
            return null;
        }
        return result;
    }

    public static void addInfo(String title) {
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, title, null));
    }

    public static void addWarn(String title) {
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_WARN, title, null));
    }

    public static void addError(String title) {
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, title, null));
    }

    public static void addFatal(String title) {
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_FATAL, title, null));
    }

    public static String getHttpSessionAttributeCaptcha(String attribute) {
        HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
        return (String) request.getSession(true).getAttribute(attribute);
    }

    public static String getStringApplicationParameter(String name) {
        return FacesContext.getCurrentInstance().getExternalContext().getInitParameter(name);
    }
    
    public static Object getApplicationBean(String name) {
        return FacesContext.getCurrentInstance().getExternalContext().getApplicationMap().get(name);
    }
    
    public static Integer getIntegerApplicationParameter(String name) {
        Integer result;        
        String s = FacesContext.getCurrentInstance().getExternalContext().getInitParameter(name);
        if (s == null) {
            return null;
        }
        try {
            result = Integer.parseInt(s);
        } catch (Exception ex) {
            return null;
        }
        return result;
    }

    public static Integer getIntegerSessionParameter(String name) {
        Integer result;
        Object o = FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(name);
        if (o == null) {
            return null;
        }
        String s = o.toString();
        if (s == null) {
            return null;
        }
        try {
            result = Integer.parseInt(s);
        } catch (Exception ex) {
            return null;
        }
        return result;
    }

    public static String getRealPath(String path) {
        return FacesContext.getCurrentInstance().getExternalContext().getRealPath(path);
    }

    public static void notCache() {
        HttpServletResponse response = (HttpServletResponse) FacesContext.getCurrentInstance().getExternalContext().getResponse();
        response.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
        response.setHeader("Pragma", "no-cache"); //HTTP 1.0
        response.setDateHeader("Expires", 0);
    }

    public static List<String> convertirListString(String s) {
        ArrayList<String> result = new ArrayList();
        StringTokenizer st = new StringTokenizer(s, "|");
        while (st.hasMoreTokens()) {
            result.add(st.nextToken());
        }
        return result;
    }
    
    public static List<String> convertirListStringBySpace(String s) {
        ArrayList<String> result = new ArrayList();
        StringTokenizer st = new StringTokenizer(s, " ");
        while (st.hasMoreTokens()) {
            result.add(st.nextToken());
        }
        return result;
    }

    public static String textToMD5(String text) {
        String output = null;
        try {
            byte[] textBytes = text.getBytes();
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(textBytes);
            byte[] codigo = md.digest();
//            output = Base64.encodeBase64String(codigo);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return output;
    }

    public static String textToMD5Hex(String password) {
        byte[] source;
        try {
            //Get byte according by specified coding.
            source = password.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            source = password.getBytes();
        }
        String result = null;

        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(source);
            //The result should be one 128 integer
            byte temp[] = md.digest();
            char str[] = new char[16 * 2];
            int k = 0;
            for (int i = 0; i < 16; i++) {
                byte byte0 = temp[i];
                str[k++] = hexDigits[byte0 >>> 4 & 0xf];
                str[k++] = hexDigits[byte0 & 0xf];
            }
            result = new String(str);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    public static boolean hasSpecialCharacter(String cadena) {
        String expresion = "[a-zA-Zñ0-9\\-_\\s]+";
        Pattern pattern = Pattern.compile(expresion);
        Matcher matcher = pattern.matcher(cadena);
        if (!matcher.matches()) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean cumpleExpresionRegular(String expresionRegular, String cadena) {
        Pattern pattern = Pattern.compile(expresionRegular);
        Matcher matcher = pattern.matcher(cadena);
        if (!matcher.matches()) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isNameCorrect(String cadena) {
        cadena = cadena.toLowerCase();
        for (int i = 0; i < cadena.length(); i++) {
            char c = cadena.charAt(i);
            if ((c < 'a') || (c > 'z')) {
                if ((c != '_') && (c != '-')) {
                    return false;
                }
            }
        }
        return true;
    }

    public static String getFechaFormateada(Date fecha) {
        if (fecha == null) {
            return "";
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fecha);
        int day = calendar.get(Calendar.DATE);
        String s_day = "" + day;
        if (s_day.length() <= 1) {
            s_day = "0" + s_day;
        }
        int month = calendar.get(Calendar.MONTH) + 1;
        String s_month = "" + month;
        if (s_month.length() <= 1) {
            s_month = "0" + s_month;
        }
        int year = calendar.get(Calendar.YEAR);
        String result = s_day + "/" + s_month + "/" + year;
        return result;
    }

    public static List<Integer> transformarPath(String path) {
        ArrayList<Integer> result = new ArrayList<>();
        if (path == null || path.equals("null")) {
            return result;
        }
        StringTokenizer st = new StringTokenizer(path, "|");
        while (st.hasMoreTokens()) {
            String s = st.nextToken();
            Integer id = Integer.parseInt(s);
            result.add(id);
        }
        return result;
    }
    
    public static String testStringEmpty(String s, boolean trim) {
        if (s == null) {
            return null;
        }
        if (s.trim().equals("")) {
            return null;
        }
        if (trim) {
            return s.trim();
        }
        return s;
    }
}
