package es.imib.persistence.daos;

import es.imib.persistence.Course;
import java.io.Serializable;
import java.util.List;
import java.util.logging.Level;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.eclipse.persistence.config.HintValues;
import org.eclipse.persistence.config.QueryHints;

public class CourseDAO implements Serializable {

    private static CourseDAO courseDAO;

    public static CourseDAO getCourseDAO() {
        if (courseDAO == null) {
            courseDAO = new CourseDAO();
        }
        return courseDAO;
    }

    protected CourseDAO() {
    }

    protected EntityManager getEntityManager() {
        return EntityManagerHelper.getEntityManager();
    }

    public Course findById(Long id) {
        try {
            Course instance = getEntityManager().find(Course.class, id);
            if (instance != null) {
                getEntityManager().refresh(instance);
            }
            return instance;
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Course> findByProperty(String propertyName, final Object value) {
        try {
            final String queryString = "select model from Course model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find by property name failed",
                    Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Course> findAll() {
        try {
            final String queryString = "select model from Course model ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    public List<Course> findByIdentifiers(List<Long> identifiers) {
        try {
            String queryString = "select distinct model from Course model where model.diagnosisNdx IN " + UtilDAO.getQueryWithMultipleLong(identifiers);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Course> findAll(String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            String queryString = "select model from Course model ";
            queryString += UtilDAO.sortColumname(sortColumnName, ascending);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public Long countAll() {
        try {
            String queryString = "select count(model) from Course model ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Course> findByProperty(String propertyName, final Object value, String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            final String queryString = "select model from Course model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find by property name failed",
                    Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public Long countByProperty(String propertyName, final Object value) {
        try {
            String queryString = "select count(model) from Course model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    @SuppressWarnings("unchecked")
    public List<Course> findByPatient(Long patient, String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            String queryString = "select model from Course model where model.patientNdx = :patient ";
            queryString += UtilDAO.sortColumname(sortColumnName, ascending);
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("patient", patient);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    @SuppressWarnings("unchecked")
    public Long countByPatient(Long patient) {
        try {
            String queryString = "select count(model) from Course model where model.patientNdx = :patient ";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("patient", patient);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);            
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
}
