package es.imib.persistence.daos;

import es.imib.persistence.MedicationSubstance;
import java.io.Serializable;
import java.util.List;
import java.util.logging.Level;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.eclipse.persistence.config.HintValues;
import org.eclipse.persistence.config.QueryHints;

public class MedicationSubstanceDAO implements Serializable {

    private static MedicationSubstanceDAO medicationSubstanceDAO;

    public static MedicationSubstanceDAO getMedicationSubstanceDAO() {
        if (medicationSubstanceDAO == null) {
            medicationSubstanceDAO = new MedicationSubstanceDAO();
        }
        return medicationSubstanceDAO;
    }

    protected MedicationSubstanceDAO() {
    }

    protected EntityManager getEntityManager() {
        return EntityManagerHelper.getEntityManager();
    }

    public MedicationSubstance findById(Long id) {
        try {
            MedicationSubstance instance = getEntityManager().find(MedicationSubstance.class, id);
            if (instance != null) {
                getEntityManager().refresh(instance);
            }
            return instance;
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<MedicationSubstance> findByProperty(String propertyName, final Object value) {
        try {
            final String queryString = "select model from MedicationSubstance model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find by property name failed",
                    Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<MedicationSubstance> findAll() {
        try {
            final String queryString = "select model from MedicationSubstance model ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    public List<MedicationSubstance> findByIdentifiers(List<Long> identifiers) {
        try {
            String queryString = "select distinct model from MedicationSubstance model where model.diagnosisNdx IN " + UtilDAO.getQueryWithMultipleLong(identifiers);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<MedicationSubstance> findAll(String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            String queryString = "select model from MedicationSubstance model ";
            queryString += UtilDAO.sortColumname(sortColumnName, ascending);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public Long countAll() {
        try {
            String queryString = "select count(model) from MedicationSubstance model ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<MedicationSubstance> findByProperty(String propertyName, final Object value, String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            final String queryString = "select model from MedicationSubstance model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find by property name failed",
                    Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public Long countByProperty(String propertyName, final Object value) {
        try {
            String queryString = "select count(model) from MedicationSubstance model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    @SuppressWarnings("unchecked")
    public List<MedicationSubstance> findByTherapy(Long therapy, String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            String queryString = "select model from MedicationSubstance model where model.massnahmeNdx = :therapy ";
            queryString += UtilDAO.sortColumname(sortColumnName, ascending);
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("therapy", therapy);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    @SuppressWarnings("unchecked")
    public Long countByPatient(Long therapy) {
        try {
            String queryString = "select count(model) from MedicationSubstance model where model.massnahmeNdx = :therapy ";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("therapy", therapy);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);            
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
}
