package es.imib.persistence.daos;

import es.imib.persistence.Patient;
import java.io.Serializable;
import java.util.List;
import java.util.logging.Level;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.eclipse.persistence.config.HintValues;
import org.eclipse.persistence.config.QueryHints;

public class PatientDAO implements Serializable {

    private static PatientDAO patientDAO;

    public static PatientDAO getPatientDAO() {
        if (patientDAO == null) {
            patientDAO = new PatientDAO();
        }
        return patientDAO;
    }

    protected PatientDAO() {
    }

    protected EntityManager getEntityManager() {
        return EntityManagerHelper.getEntityManager();
    }

    public Patient findById(Long id) {
        try {
            Patient instance = getEntityManager().find(Patient.class, id);
            if (instance != null) {
                getEntityManager().refresh(instance);
            }
            return instance;
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Patient> findByProperty(String propertyName, final Object value) {
        try {
            final String queryString = "select model from Patient model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find by property name failed",
                    Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Patient> findAll() {
        try {
            final String queryString = "select model from Patient model ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    public List<Patient> findByIdentifiers(List<Long> identifiers) {
        try {
            String queryString = "select distinct model from Patient model where model.diagnosisNdx IN " + UtilDAO.getQueryWithMultipleLong(identifiers);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Patient> findAll(String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            String queryString = "select model from Patient model ";
            queryString += UtilDAO.sortColumname(sortColumnName, ascending);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    @SuppressWarnings("unchecked")
    public List<Patient> findAllWithDiagnosis(String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            String queryString = "select distinct(model) from Patient model, Diagnosis d where model.patientNdx = d.patientNdx ";
            queryString += UtilDAO.sortColumname(sortColumnName, ascending);
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public Long countAll() {
        try {
            String queryString = "select count(model) from Patient model ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    @SuppressWarnings("unchecked")
    public Long countAllWithDiagnosis() {
        try {
            String queryString = "select count(distinct(model)) from Patient model, Diagnosis d where model.patientNdx = d.patientNdx ";
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public List<Patient> findByProperty(String propertyName, final Object value, String sortColumnName, Boolean ascending, final int... rowStartIdxAndCount) {
        try {
            final String queryString = "select model from Patient model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            if (rowStartIdxAndCount != null && rowStartIdxAndCount.length > 0) {
                int rowStartIdx = Math.max(0, rowStartIdxAndCount[0]);
                if (rowStartIdx > 0) {
                    query.setFirstResult(rowStartIdx);
                }
                if (rowStartIdxAndCount.length > 1) {
                    int rowCount = Math.max(0, rowStartIdxAndCount[1]);
                    if (rowCount > 0) {
                        query.setMaxResults(rowCount);
                    }
                }
            }
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find by property name failed",
                    Level.SEVERE, re);
            throw re;
        }
    }

    @SuppressWarnings("unchecked")
    public Long countByProperty(String propertyName, final Object value) {
        try {
            String queryString = "select count(model) from Patient model where model."
                    + propertyName + "= :propertyValue";
            Query query = getEntityManager().createQuery(queryString);
            query.setParameter("propertyValue", value);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return (Long) query.getSingleResult();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
    
    public List<Object[]> findGroupBy(String groupBy) {
        try {
            String queryString = "select model." + groupBy + ", count(distinct(model)) from Patient model, Diagnosis d "
                    + " where d.patientNdx = model.patientNdx "
                    + " group by model." + groupBy;
            Query query = getEntityManager().createQuery(queryString);
            query.setHint(QueryHints.REFRESH, HintValues.TRUE);
            return query.getResultList();
        } catch (RuntimeException re) {
            EntityManagerHelper.log("find all failed", Level.SEVERE, re);
            throw re;
        }
    }
}
