package es.imib.semantic;

import es.imib.cccf.web.application.OGIFilter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.query.BindingSet;
import org.openrdf.query.GraphQuery;
import org.openrdf.query.GraphQueryResult;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.semanticweb.owlapi.model.OWLClass;

public class RDFQuery implements Serializable {

    protected Repository repository;

    public RDFQuery(Repository repository) {
        this.repository = repository;
    }

    public TupleQueryResult runQuery(RepositoryConnection connection, String query) {
        try {
            TupleQuery tupleQuery = connection.prepareTupleQuery(QueryLanguage.SPARQL, query);
            TupleQueryResult result = tupleQuery.evaluate();
            return result;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }
    
    public GraphQueryResult runGraphQuery(RepositoryConnection connection, String query) {
        try {
            GraphQuery graphQuery = connection.prepareGraphQuery(QueryLanguage.SPARQL, query);
            GraphQueryResult result = graphQuery.evaluate();
            return result;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    public List<Tuple> runQueryTuplesById(String id, String propertyId, List<String> owlClasses) {
        List<Tuple> tuples = new ArrayList<>();
        String queryOWLClasses = "";
        if (owlClasses != null && !owlClasses.isEmpty()) {
            queryOWLClasses = "{";
            queryOWLClasses += "{?s rdf:type <" + owlClasses.get(0) + ">} ";
            for (int i = 1; i < owlClasses.size(); i++) {
                queryOWLClasses += " UNION {?s rdf:type <" + owlClasses.get(i) + ">}";
            }
            queryOWLClasses += "}";
        }
        String query = "SELECT ?s ?p ?o WHERE {"
                + "{?s ?p ?o} . "
                + "{?s <" + propertyId + "> ?c . FILTER (?c = '" + id + "'^^<http://www.w3.org/2001/XMLSchema#string>)} . "
                + queryOWLClasses
                + "}";
        try {
            RepositoryConnection connection = repository.getConnection();
            try {
                TupleQueryResult result = runQuery(connection, query);
                try {
                    while (result.hasNext()) {
                        BindingSet bs = result.next();
                        Value subject = bs.getValue("s");
                        Value predicate = bs.getValue("p");
                        Value object = bs.getValue("o");
                        Tuple t = new Tuple(subject, predicate, object);
                        tuples.add(t);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                } finally {
                    result.close();
                }                    
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                connection.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return tuples;
    }
    
    public List<Tuple> runQueryTuplesById(String id, String propertyId, String owlClasses) {
        List<String> st = new ArrayList<>();
        st.add(owlClasses);
        return runQueryTuplesById(id, propertyId, st);
    }
    
    public List<Tuple> runQueryTuplesByURI(URI uri) {
        List<Tuple> tuples = new ArrayList<>();        
        String query = "SELECT ?s ?p ?o WHERE {"
                + "{<" + uri.toString() + "> ?p ?o} "
                + "}";
        try {
            RepositoryConnection connection = repository.getConnection();
            try {
                TupleQueryResult result = runQuery(connection, query);
                try {
                    while (result.hasNext()) {
                        BindingSet bs = result.next();
                        Value subject = bs.getValue("s");
                        Value predicate = bs.getValue("p");
                        Value object = bs.getValue("o");
                        Tuple t = new Tuple(subject, predicate, object);
                        tuples.add(t);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                } finally {
                    result.close();
                }                    
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                connection.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return tuples;
    }
    
    public List<Tuple> runQueryTuplesByURI(List<URI> uris) {
        List<Tuple> tuples = new ArrayList<>();        
        String queryAux;
        if (uris == null || uris.isEmpty()) {
            return tuples;
        }
        queryAux = ". FILTER(?s = <" + uris.get(0).toString() + ">";
        for (int i = 1; i < uris.size(); i++) {
            queryAux += " || ?s = <" + uris.get(i).toString() + ">";
        }
        queryAux += ")";
        String query = "SELECT ?s ?p ?o WHERE { {?s ?p ?o " + queryAux + "}"
                + "}";
        try {
            RepositoryConnection connection = repository.getConnection();
            try {
                TupleQueryResult result = runQuery(connection, query);
                try {
                    while (result.hasNext()) {
                        BindingSet bs = result.next();
                        Value subject = bs.getValue("s");
                        Value predicate = bs.getValue("p");
                        Value object = bs.getValue("o");
                        Tuple t = new Tuple(subject, predicate, object);
                        tuples.add(t);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                } finally {
                    result.close();
                }                    
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                connection.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return tuples;
    }
    
    public List<URI> findURIByClass(List<OWLClass> ocs, List<OGIFilter> filters, int... parameters) {
        List<URI> uris = new ArrayList<>();        
        String queryAux;
        if (ocs == null || ocs.isEmpty()) {
            return uris;
        } 
        OWLClass oc = ocs.get(0);
        queryAux = "{?s rdf:type <" + oc.toStringID() + ">} ";
        for (int i = 1; i < ocs.size(); i++) {
            OWLClass c = ocs.get(i);
            queryAux += "UNION {?s rdf:type <" + c.toStringID() + ">} ";
        }
        String filtersStr = "";
        if (filters != null && !filters.isEmpty()) {
            filtersStr = " . {" + filters.get(0).getFilter();
            for (int i = 1; i < filters.size(); i++) {
                OGIFilter of = filters.get(i);
                if (of.isCondition()) {
                    filtersStr += " UNION " + of.getFilter();
                } else {
                    filtersStr += " . " + of.getFilter();
                }
                
            }
            filtersStr += "}";
        }        
        String query = "SELECT DISTINCT ?s WHERE {"
                + queryAux
                + filtersStr
                + "}";
        System.out.println(query);
        if (parameters != null) {
            if (parameters.length == 2) {
                int offset = parameters[0];                
                int size = parameters[1];  
                query += " offset " + offset + " limit " + size;
            }
        }
        try {
            RepositoryConnection connection = repository.getConnection();
            try {
                TupleQueryResult result = runQuery(connection, query);
                try {
                    while (result.hasNext()) {
                        BindingSet bs = result.next();
                        Value subject = bs.getValue("s");                                                
                        uris.add((URI) subject);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                } finally {
                    result.close();
                }                    
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                connection.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return uris;
    }
    
    public Integer countURIByClass(List<OWLClass> ocs, List<OGIFilter> filters) {        
        String queryAux;
        if (ocs == null || ocs.isEmpty()) {
            return 0;
        } 
        OWLClass oc = ocs.get(0);
        queryAux = "{?s rdf:type <" + oc.toStringID() + ">} ";
        for (int i = 1; i < ocs.size(); i++) {
            OWLClass c = ocs.get(i);
            queryAux += "UNION {?s rdf:type <" + c.toStringID() + ">} ";
        }
        String filtersStr = "";
        if (filters != null && !filters.isEmpty()) {
            filtersStr = " . {" + filters.get(0).getFilter();
            for (int i = 1; i < filters.size(); i++) {
                OGIFilter of = filters.get(i);
                if (of.isCondition()) {
                    filtersStr += " UNION " + of.getFilter();
                } else {
                    filtersStr += " . " + of.getFilter();
                }
                
            }
            filtersStr += "}";
        }
        String query = "SELECT count(DISTINCT ?s) WHERE {"
                + queryAux
                + filtersStr
                + "}";
        try {
            RepositoryConnection connection = repository.getConnection();
            try {
                TupleQueryResult result = runQuery(connection, query);
                try {
                    List<String> list = result.getBindingNames();
                    while (result.hasNext()) {
                        BindingSet bs = result.next();
                        Value count = bs.getValue(list.get(0));
                        try {
                            return Integer.parseInt(count.stringValue());
                        } catch (Exception ex) {
                            
                        }
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                } finally {
                    result.close();
                }                    
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                connection.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return 0;
    }
}
